function ret = qrcode_read_target(img_plate, i_axis, j_axis)
    % ret = qrcode_read_target(img_plate)
    % 
    % read chessboard qr code from calibration image
    % return coordinates 

    %% define coordinate axes for image if not supplied by user
    n_i                 = size(img_plate,1);
    n_j                 = size(img_plate,2);
    if nargin < 2
        i_axis          = 1 : n_i;
    end
    if nargin < 3
        j_axis          = 1 : n_j;
    end
    
    %% assign default empty structure for output
    ret                 = struct();
    % grid parameters
    ret.grid_dimension  = [0 0];
    ret.grid_spacing    = 1;
    ret.u_lim           = [0 1];
    ret.v_lim           = [0 1];
    ret.grid_units      = [];
    % chessboard corners
    ret.n_markers       = 0;
    ret.uv_markers      = zeros(2,0);
    ret.im_markers      = zeros(2,0);
    % QR code marker corners
    ret.n_qrpts         = 0;
    ret.uv_qrpts        = zeros(2,0);
    ret.im_qrpts        = zeros(2,0);
    % masks for region also need transformation and scale applying
    ret.roimask         = polyshape(i_axis([1 1 end end]), j_axis([1 end end 1]));
    ret.qrmask          = polyshape(i_axis([1 1 end end]), j_axis([1 end end 1]));

    %% read details of this calibration plate from the QR code
    [msg, ~, im_qrpts] = readBarcode(img_plate.', 'QR-CODE');
    if isempty(im_qrpts)
        warning('Unable to read QR code');
        return;
    end
    % extract coordinates of QR code corners
    im_qrpts            = im_qrpts.';% first row i, second row j
    n_qrpts             = size(im_qrpts, 2);
    if n_qrpts ~= 4
        warning('Failed to read QR code: four corners must be identified');
        return;
    end
    
    %% extract (u,v) coordinates of QR code fiduciary marker
    qrdata          = qrcode_read_string(msg);
    grid_dimension  = qrdata.dimension;
    grid_spacing    = qrdata.spacing(1);
    assert(grid_spacing == qrdata.spacing(2), 'only square chessboard tiles are supported');

    %% determine affine transformation matrix
    % the QR code occupies one chessboard square
    % so we determine the corners of that square based on the affine
    % im_qrpts ~ M_1 * uv_qrpts

    % the u,v coordinates of the qr corners are at fractional positions
    % over the chessboard square. 
    % here we reverse engineer their true positions
    qrcode          = ~encode_qr(msg,[1 1]).';
    qrcode_width    = size(qrcode,1);
    [~,~,im_qrpts_rel] = readBarcode(qrcode.','QR-CODE');
    % normalise to size of QR code so that (0,0) is bottom left and (1,1)
    % is top right of code
    uv_qrpts_rel    = (im_qrpts_rel.' - 1.5)/(qrcode_width-1); 
    uv_qrpts        = qrdata.origin(:) + uv_qrpts_rel;
    
    % apply permutation
    % for some reason, the order of points identified by readBarcode is not
    % always consistent. this ordering [3 2 1 4] seems to work for now...
    uv_qrpts        = uv_qrpts(:, [3 2 1 4]);

    % determine affine transformation 
    % [y1 y2 1] ~ M1 [u v 1]
    imt_qrpts       = [im_qrpts; ones(1, n_qrpts)];
    uvt_qrpts       = [uv_qrpts; ones(1, n_qrpts)];
    M1              = dlt33(imt_qrpts, uvt_qrpts);

    % determine corners of QR checkerboard
    uv_qrcorners    = [0 0 1 1; 0 1 1 0] + qrdata.origin(:);
    im_qrcorners    = M1 * [uv_qrcorners; 1 1 1 1];
    im_qrcorners    = im_qrcorners(1:2,:)./im_qrcorners(3,:);
    qrmask          = polyshape(im_qrcorners(1,:), im_qrcorners(2,:));

    % determine corners of mask
    u_lim           = [-0.5 0.5] * grid_dimension(1);
    v_lim           = [-0.5 0.5] * grid_dimension(2);
    uv_roicorners   = [u_lim([1 1 2 2]) ; v_lim([1 2 2 1])] ...
                    + [1 1 -1 -1; 1 -1 -1 1]*0.5;
    im_roicorners   = M1 * [uv_roicorners; 1 1 1 1];
    im_roicorners   = im_roicorners(1:2,:)./im_roicorners(3,:);
    roimask         = polyshape(im_roicorners(1,:), im_roicorners(2,:));

    % mask image to checkerboard region
    img_mask        = poly2mask(im_roicorners(2,:), im_roicorners(1,:), n_i, n_j);
    
    %% define u and v coordinate system
    n_u             = size(img_plate, 1);
    n_v             = size(img_plate, 2);
    u_axis          = linspace(-grid_dimension(1)/2, grid_dimension(1)/2, n_u)*grid_spacing;
    v_axis          = linspace(-grid_dimension(2)/2, grid_dimension(2)/2, n_v)*grid_spacing;

    %% detect chessboard corners
    % apply ROI mask
    img_cropped     = img_mask .* img_plate;

    [im_markers, bsize] = detectCheckerboardPoints(img_cropped.');
    if isempty(im_markers)
	    warning('no chessboard corners found, try again with a different threshold');
        return;
    end
    % image coordinates of chessboard points
    im_markers  = im_markers.';% 1st row i, second row j
    % world coordinates of chessboard points
    % note this is a different coordinate system to (u,v) system defined by
    % QR code marker. It is nominally the same scale
    UV_markers  = generateCheckerboardPoints(bsize, 1).';
    % eliminate NaN data and clear outliers
    % sometimes, we get chessboard edges identified inside the QR code area
    % we can identify these and eliminate them
    outsideroi  = isinterior(qrmask, im_markers(1,:), im_markers(2,:));
    bad         = any(isnan(im_markers), 1) | outsideroi(:).';
    im_markers(:,bad)= [];
    UV_markers(:,bad)= [];
    n_markers   = size(im_markers,2);
    % determine affine transformation 
    % [y1; y2; 1] ~ M2 [U; V; 1]
    UVt_markers = [UV_markers; ones(1,n_markers)];
    imt_markers = [im_markers; ones(1,n_markers)];
    M2          = dlt33(imt_markers, UVt_markers);

    % determine (u,v) coordinates of markers based on 
    % [y1; y2; 1] ~ M2 [U; V; 1] ~ M1 [u; v; 1]
    % [u; v; 1] ~ inv(M1) * M2 * [U; V; 1]
    M12         = M1 \ M2;
   
    % the transformation M12 should be of the form
    % [+-1 0 u0; 0 +-1 v0; 0 0 1]
    % where u0 and v0 are integer spacings
    M12_orient  = M12(1:2, 1:2) / M12(3,3);
    A           = round(M12_orient); % hackish way of approximating rotation matrix
    b           = inv(M2) * imt_qrpts;
    b           = b(1:2,:) ./ b(3,:);
    Delta       = mean(b - A\uv_qrpts, 2); % best fit Delta
    % round Delta to a half integer fraction
    Delta       = round(Delta*2)/2;
    M12_rounded = [inv(A) -A*Delta; 0 0 1];

    % here we do some rounding to estimate the correct transformation
    %M12         = M12 / M12(3,3);
    %M12_rounded = [round(M12(1:2,1:2)) round(M12(1:2,3)*2)/2; 0 0 1];
    uvt_markers = M12_rounded * UVt_markers;
    uv_markers  = uvt_markers(1:2,:) ./ uvt_markers(3,:);
    uv_markers  = round(uv_markers*2)/2;

    %% fill structure
    % at this stage, we convert from chessboard coordinate system to
    % physical units.
    % also apply conversion from row/col coordinates to image coordinates
    % given by i_axis and j_axis
    di                  = mean(diff(i_axis));
    dj                  = mean(diff(j_axis));
    ij0                 = [i_axis(1); j_axis(1)];

    ret                 = struct();
    % grid parameters
    ret.grid_dimension  = grid_dimension;
    ret.grid_spacing    = grid_spacing;
    ret.u_lim           = u_lim * grid_spacing;
    ret.v_lim           = v_lim * grid_spacing;
    ret.grid_units      = qrdata.units;

    % chessboard corners
    ret.n_markers       = n_markers;
    ret.uv_markers      = uv_markers * grid_spacing;
    ret.im_markers      = diag([di dj])*im_markers + ij0;
    
    % QR code marker corners
    ret.n_qrpts         = n_qrpts;
    ret.uv_qrpts        = uv_qrpts * grid_spacing;
    ret.im_qrpts        = diag([di dj]) * im_qrpts + ij0;

    % masks for region also need transformation and scale applying
    ret.roimask         = translate(scale(roimask, [di dj]), ij0.');
    ret.qrmask          = translate(scale(qrmask , [di dj]), ij0.');

    return;
    %% debug: show points identified on target
    figure(1);
    clf;
    imagesc(img_plate');
    
    hold on;
    plot(roimask);
    plot(qrmask);

    plot(im_qrpts(1,:), im_qrpts(2,:), 'gs');
    for m = 1 : n_qrpts
        str=sprintf('%d) %0.1f, %0.1f', m, uv_qrpts(:,m));
        text(im_qrpts(1,m), im_qrpts(2,m), str, 'color', 'g', 'fontsize', 16);
    end

    if ~isempty(im_markers)
        plot(im_markers(1,:), im_markers(2,:), 'ro-');
        for m = 1 : n_markers
            str=sprintf('%0.0f, %0.0f', uv_markers(:,m));
            text(im_markers(1,m), im_markers(2,m), str, 'color', 'r', 'fontsize', 16);
        end
    end
    
    colormap gray;
    axis equal tight ij;
    xlim([0 size(img_plate,1)]);
    ylim([0 size(img_plate,2)]);

    %% debug plot: show points identified on QR code
    figure(3);
    clf;
    u_qr = linspace(0,1,qrcode_width) + qrdata.origin(1);
    v_qr = linspace(0,1,qrcode_width) + qrdata.origin(2);
    imagesc(u_qr,v_qr,qrcode');
    hold on;
    plot(uv_qrpts(1,:), uv_qrpts(2,:), 'ro');
    for m = 1 : n_qrpts
        str=sprintf('%0.1f,%0.1f', uv_qrpts(:,m));
        text(uv_qrpts(1,m), uv_qrpts(2,m), str, 'color', 'r');
    end
    axis equal tight xy;
    colormap gray;

