function [C_x, C_y, C_z] = curl_matrix(x, y, z, i, j, k)
	%% construct curl operator
	% result is 3 separate matrices
	% representing components of curl in 3 directions
	% applied at points x(i), y(j), z(k) on regularly spaced grid
	% defined by x, y, z, and i, j, k
	
	%% sizes of stuff
	dx							= x(2) - x(1);
	dy							= y(2) - y(1);
	dz							= z(2) - z(1);
	Nx							= length(x);
	Ny							= length(y);
	Nz							= length(z);
	Ni							= length(i);
	Nj							= length(j);
	Nk							= length(k);
	N_curl					= Ni * Nj * Nk;
	N_tot						= Nx * Ny * Nz;
	S							= [3 Nx Ny Nz];

	%% generate grid of indices
	[imat, jmat, kmat]	= ndgrid(i, j, k);
	imat						= imat(:);
	jmat						= jmat(:);
	kmat						= kmat(:);

	%% generate linear indices
	% linear index of points (i+-1,j+-1,k+-1) in velocity array
	% which is 3 x Nx x Ny x Nz
	u_ijk						= zeros(N_curl, 3, 3, 3);
	v_ijk						= zeros(N_curl, 3, 3, 3);
	w_ijk						= zeros(N_curl, 3, 3, 3);
	for dk = -1 : 1
		for dj = -1 : 1
			for di = -1 : 1
				u_ijk(:, di+2, dj+2, dk+2) = sub2ind(S, 1*ones(N_curl,1), imat + di, jmat + dj, kmat + dk);
				v_ijk(:, di+2, dj+2, dk+2) = sub2ind(S, 2*ones(N_curl,1), imat + di, jmat + dj, kmat + dk);
				w_ijk(:, di+2, dj+2, dk+2) = sub2ind(S, 3*ones(N_curl,1), imat + di, jmat + dj, kmat + dk);
			end
		end
	end

	%% construct curl operator programatically
	% the curl operator is the same for each component, barring a permutation
	% of the velocity components. 
	% The curl in a particular direction is the circulation over a patch 
	% divided by area of said patch
	% . < . < .
	% v       ^
	% .   o   .
	% v       ^
	% . > . > .
	%

	curl_kernel_u1			= [+1  0 -1;+2  0 -2; +1  0 -1]/2;
	curl_kernel_u2			= [-1 -2 -1; 0  0  0; +1 +2 +1]/2;
	ivec_curlx				= zeros(18, N_curl);
	jvec_curlx				= zeros(18, N_curl);
	svec_curlx				= zeros(18, N_curl);
	ivec_curly				= zeros(18, N_curl);
	jvec_curly				= zeros(18, N_curl);
	svec_curly				= zeros(18, N_curl);
	ivec_curlz				= zeros(18, N_curl);
	jvec_curlz				= zeros(18, N_curl);
	svec_curlz				= zeros(18, N_curl);
	for idx = 1 : N_curl
		%% z component of curl
		% u1 is in x direction, u2 is in y direction
		iu_local				= u_ijk(idx, :, :, 2);
		iv_local				= v_ijk(idx, :, :, 2);
		ivec_curlz(:, idx)= [iu_local(:); iv_local(:)];
		jvec_curlz(:, idx)= idx;
		svec_curlz(:, idx)= [+curl_kernel_u1(:)*dx; +curl_kernel_u2(:)*dy] / (4*dx*dy);

		%% y component of curl
		% u1 is in -x direction u2 is in +z direction
		iu_local				= u_ijk(idx, :, 2, :);
		iw_local				= w_ijk(idx, :, 2, :);
		ivec_curly(:, idx)= [iu_local(:); iw_local(:)];
		jvec_curly(:, idx)= idx;
		svec_curly(:, idx)= [-curl_kernel_u1(:)*dx; +curl_kernel_u2(:)*dz] / (4*dx*dz);

		%% x component of curl
		% u1 is in y direction, u2 is in z direction
		iv_local				= v_ijk(idx, 2, :, :);
		iw_local				= w_ijk(idx, 2, :, :);
		ivec_curlx(:, idx)= [iv_local(:); iw_local(:)];
		jvec_curlx(:, idx)= idx;
		svec_curlx(:, idx)= [+curl_kernel_u1(:)*dy; +curl_kernel_u2(:)*dz] / (4*dy*dz);
	end
	
	%% populate sparse matrices
	nz		= svec_curlz ~= 0;
	C_z	= sparse(ivec_curlz(nz), jvec_curlz(nz), svec_curlz(nz), 3*N_tot, N_curl);
	nz		= svec_curly ~= 0;
	C_y	= sparse(ivec_curly(nz), jvec_curly(nz), svec_curly(nz), 3*N_tot, N_curl);
	nz		= svec_curlx ~= 0;
	C_x	= sparse(ivec_curlx(nz), jvec_curlx(nz), svec_curlx(nz), 3*N_tot, N_curl);
end