function [ux_filt, uy_filt, uz_filt] = divfree_o2(ux, uy, uz, x, y, z, Lambda)
	% [ux_filt, uy_filt, uz_filt] = divfree(ux, uy, uz, x, y, z, Lambda)
	%
	% return the closest approximation to the velocity field specified by
	% the 3D arrays u = (ux,uy,uz) that has zero divergence
	% 
	% the x, y and z axes correspond to the first, second and third
	% dimensions of the arrays
	% 
	% ux_filt, uy_filt and uz_filt are divergence free vector fields
	%
	% Lambda is a regularisation parameter that trades off closeness of fit
	% against the divergence free condition
	%
	% the divergence free criterion is only applied to the interior points
	% of ux, uy and uz. Divergence is calculated using first order central
	% differences
	
	if nargin > 3
		Nx				= length(x);
		Ny				= length(y);
		Nz				= length(z);
		assert(isequal(size(ux), [Nx Ny Nz]));
		assert(isequal(size(uy), [Nx Ny Nz]));
		assert(isequal(size(uz), [Nx Ny Nz]));
	else
		Nx				= size(ux,1);
		Ny				= size(ux,2);
		Nz				= size(ux,3);
		x				= 1 : Nx;
		y				= 1 : Ny;
		z				= 1 : Nz;
		Lambda		= 1;
	end
	
	% construct the divergence constraint
	u_noisy		= shiftdim(cat(4, ux, uy, uz), 3);
	[ic,jc,kc]	= ndgrid(2:Nx-1,2:Ny-1,2:Nz-1);
	C_divergence= oper_div_cfd_o2(x, y, z, ic, jc, kc);
	d_noisy		= C_divergence * u_noisy(:);
	
	% solve in least squares sense using Tikhonov regularisation
	% construct a vector u_error such that 
	% ||C * ue|| + Lambda*||ue|| is minimised
	CtC			= C_divergence.' * C_divergence;
	GtG			= Lambda^2 * speye(3*Nx*Ny*Nz, 3*Nx*Ny*Nz);
	Ctd			= C_divergence.' * d_noisy;
	u_error		= (CtC + GtG)\Ctd;
	
	u_filt		= u_noisy(:) - u_error;
	u_filt		= reshape(u_filt, [3 Nx Ny Nz]);
	
	% recover parameters
	ux_filt		= squeeze(u_filt(1,:,:,:));
	uy_filt		= squeeze(u_filt(2,:,:,:));
	uz_filt		= squeeze(u_filt(3,:,:,:));
end