function C_grad = gradient_matrix(x, y, z, c, Nc)
	%% construct grad operator
	% for an array Umat which is of size Nc x Nx x Ny x Nz
	%
	% we construct a constraint C * Umat = grad(phi)
	% where phi is the cth component of Umat, i.e. Umat(c, :,:,:)
	% 
	% we have a grid x, y, z and we want a matrix representing the application
	% of the gradient operator on a grid x(i), y(j), z(k)
	%
	% the result is a vector of dphi/dx, dphi/dy and dphi/dz
	% ordered as the flattened form of a 3 x Nx x Ny x Nz matrix
	
	%% sizes of stuff
	dx							= x(2) - x(1);
	dy							= y(2) - y(1);
	dz							= z(2) - z(1);
	Nx							= length(x);
	Ny							= length(y);
	Nz							= length(z);
	Np							= Nx * Ny * Nz;
	S							= [Nc Nx Ny Nz];

	%% generate grid of indices
	i							= 1 : Nx;
	j							= 1 : Ny;
	k							= 1 : Nz;
	[imat, jmat, kmat]	= ndgrid(i, j, k);
	imat						= imat(:);
	jmat						= jmat(:);
	kmat						= kmat(:);

	%% populate matrix
	% using fourth order accurate gradients, we may use up to 4 values of phi
	% to calcultae a gradient 
	ivec						= zeros(4, 3, Nx, Ny, Nz);
	jvec						= zeros(4, 3, Nx, Ny, Nz);
	svec						= zeros(4, 3, Nx, Ny, Nz);
	grad_kernel_idx		= cell(1,3);
	grad_kernel				= cell(1,3);
	grad_kernel_size		= zeros(1,3);
	
	for kk = 1 : Nz
		[grad_kernel{3}, grad_kernel_idx{3}, grad_kernel_size(3)] = ...
			gradient_kernel(kk, Nz, dz);	
		for jj = 1 : Ny
			[grad_kernel{2}, grad_kernel_idx{2}, grad_kernel_size(2)] = ...
				gradient_kernel(jj, Ny, dy);
			for ii = 1 : Nx
				[grad_kernel{1}, grad_kernel_idx{1}, grad_kernel_size(1)] = ...
					gradient_kernel(ii, Nx, dx);

				%% find gradients of phi over each direction m = 1,2,3
				for m = 1 : 3
					% forgive me lord, for the for/switch combination
					ksize				= grad_kernel_size(m);
					switch m
					case 1
						col_idx		= sub2ind(S, c*ones(1,ksize), ...
											grad_kernel_idx{m}, jj*ones(1,ksize),   kk*ones(1,ksize)  );
					case 2
						col_idx		= sub2ind(S, c*ones(1,ksize), ...
											ii*ones(1,ksize),   grad_kernel_idx{m}, kk*ones(1,ksize)  );
					otherwise
						col_idx		= sub2ind(S, c*ones(1,ksize), ...
											ii*ones(1,ksize),   jj*ones(1,ksize),   grad_kernel_idx{m});
					end
					row_idx			= sub2ind([3 Nx Ny Nz], m, ii, jj, kk) * ones(1,ksize);
					
					ivec(1:ksize, m, ii, jj, kk) = row_idx;
					jvec(1:ksize, m, ii, jj, kk) = col_idx;
					svec(1:ksize, m, ii, jj, kk) = grad_kernel{m};					
				end
			end
		end
	end
	nz							= svec ~= 0;
	C_grad					= sparse(ivec(nz), jvec(nz), svec(nz), 3*Nx*Ny*Nz, Nc*Nx*Ny*Nz);
end

function [kernel, idx, ksize] = gradient_kernel(i, Ni, dx)
	% choose the most appropriate kernel
	% for first order derivatives dy/di

	if i > 2 && i < Ni-1
		% fourth order kernel
		kernel		=     [-1  8 -8  1] / (12*dx);
		idx			= i + [+2 +1 -1 -2];		
	elseif i > 1 && i < Ni
		% second order kernel
		kernel		=     [+1 -1] / (2*dx);
		idx			= i + [+1 -1];
	elseif i == 1
		% first order kernel, forward differences
		kernel		=     [+1 -1] / dx;
		idx			= i + [+1  0];
	else
		% first order kernel, backwards differences
		kernel		=     [+1 -1] / dx;
		idx			= i + [0  -1];
	end
	ksize = length(kernel);
end