function [GL,i,j,s] = gradient_matrix_1d(x, missing, kernel)
	%% construct 1D gradient operator
	% for a vector phi(x), G*phi = dphi/dx
	% where elements of phi(x) may be missing at points defined by mask
	% mask is true where elements of phi are missing
	% replace missing elements with a linear interpolation of phi
	%
	% the resultant matrix G is square Nx x Nx and sparse 
	% 
	%
	% the resultant matrix GL should not have more than 8*Nx nonzero
	% elements, as each gradient uses 4 points, which may be based on as
	% many as 2 other points if interpolating/extrpolating
	
	%% sizes of stuff
	dx							= x(2) - x(1);
	Nx							= length(x);
	if nargin < 2
		missing				= false(Nx,1);
	end
	
	%% create linear interpolator matrix
	L							= zeros(Nx+4, Nx);
	ii_present				= find(~missing);
	if sum(~missing) < 2
		% too many missing points
		% return an all zero sparse matrix
		GL						= spalloc(Nx,Nx,1);
		i						= [];
		j						= [];
		s						= [];
		return;
	end
	
	for ii = -1 : Nx+2
		if ii >= 1 && ii <= Nx && ~missing(ii)
			% dealing with an interior point which is present
			L(ii+2, ii)		= 1;
		else
			% interpolation/extrapolation required
			% get the two nearest linear indices next to this point (that
			% aren't missing)
			[~,idx]			= sort(abs(ii_present - ii), 'ascend');
			ii_0				= min(ii_present(idx(1:2)));	% left point
			ii_1				= max(ii_present(idx(1:2)));	% right point
			% now we have two points: do standard linear
			% interpolation/extrapolation between those two
			delta				= (ii - ii_0) / (ii_1 - ii_0);
			L(ii+2,ii_0)	= 1 - delta;
			L(ii+2,ii_1)	= delta;
		end
	end
	
	%% populate finite difference operator matrix
	% kernel MUST be 5 point stencil
	if nargin < 3
		kernel		= [+1 -8 0 +8 -1] / (12*dx);
	end
	G_first_row		= [kernel     zeros(1,Nx-1)];
	G_first_col		= [kernel(1); zeros(Nx-1,1)];
	G					= toeplitz(G_first_col, G_first_row);
	
	%% finished matrix is G*L
	% which we wish to be sparse
	GL					= G*L;
	[i,j,s]			= find(GL);
	GL					= sparse(i,j,s,Nx,Nx);
end
