function G = gradient_matrix_3d(x_axis, y_axis, z_axis, missing)
	% construct a matrix operator G which acts on phi(:), where 
	% phi is a Nx x Ny x Nz matrix defined on a grid of x, y and z
	% specified by x_axis, y_axis and z_axis
	%
	% the result grad(:) = G * phi(:) is a 3 x Nx x Ny x Nz array
	% representing a finite difference approximation to 
	% dphi/dx, dphi/dy and dphi/dz at all points in the array
	%
	% a facility is provided to ignore missing points in phi
	% provide a logical, Nx x Ny x Nz array ``missing''
	% which contains a true value where an element of phi is missing.
	% it will be replaced with linear interpolation/extrapolation from its
	% nearest neighbours
	%
	
	Nx				= length(x_axis);
	Ny				= length(y_axis);
	Nz				= length(z_axis);
	if nargin < 4
		missing	= false(Nx, Ny, Nz);
	end
	
	%% gradient operator in x direction
	% create entries for calculating all the gradients in x
	% row by row of phi
	ivec_x		= zeros(8*Nx, Ny, Nz);
	jvec_x		= zeros(8*Nx, Ny, Nz);
	svec_x		= zeros(8*Nx, Ny, Nz);
	for kk = 1 : Nz
		for jj = 1 : Ny
			missing_row	= missing(:, jj, kk);
			[~,r,c,s]	= gradient_matrix_1d(x_axis, missing_row);
			n_nz			= length(s);
			if n_nz == 0
				continue;
			end
			row_idx		= sub2ind([3 Nx Ny Nz], 1*ones(n_nz,1), r, jj*ones(n_nz,1), kk*ones(n_nz,1));
			col_idx		= sub2ind([  Nx Ny Nz],                 c, jj*ones(n_nz,1), kk*ones(n_nz,1));
			ivec_x(1:n_nz, jj, kk) = row_idx;
			jvec_x(1:n_nz, jj, kk) = col_idx;
			svec_x(1:n_nz, jj, kk) = s;
		end
	end
	
	%% gradient operator in y direction
	ivec_y		= zeros(8*Ny, Nx, Nz);
	jvec_y		= zeros(8*Ny, Nx, Nz);
	svec_y		= zeros(8*Ny, Nx, Nz);
	for kk = 1 : Nz
		for ii = 1 : Nx
			missing_row = missing(ii, :, kk);
			[~,r,c,s]	= gradient_matrix_1d(y_axis, missing_row);
			n_nz			= length(s);
			if n_nz == 0
				continue;
			end
			row_idx		= sub2ind([3 Nx Ny Nz], 2*ones(n_nz,1), ii*ones(n_nz,1), r, kk*ones(n_nz,1));
			col_idx		= sub2ind([  Nx Ny Nz],                 ii*ones(n_nz,1), c, kk*ones(n_nz,1));
			ivec_y(1:n_nz, ii, kk) = row_idx;
			jvec_y(1:n_nz, ii, kk) = col_idx;
			svec_y(1:n_nz, ii, kk) = s;
		end
	end
	
	%% gradient operator in z direction
	ivec_z		= zeros(8*Nz, Nx, Ny);
	jvec_z		= zeros(8*Nz, Nx, Ny);
	svec_z		= zeros(8*Nz, Nx, Ny);
	for jj = 1 : Ny
		for ii = 1 : Nx
			missing_row = missing(ii, jj, :);
			[~,r,c,s]	= gradient_matrix_1d(z_axis, missing_row);
			n_nz			= length(s);
			if n_nz == 0
				continue;
			end
			row_idx		= sub2ind([3 Nx Ny Nz], 3*ones(n_nz,1), ii*ones(n_nz,1), jj*ones(n_nz,1), r);
			col_idx		= sub2ind([  Nx Ny Nz],                 ii*ones(n_nz,1), jj*ones(n_nz,1), c);
			ivec_z(1:n_nz, ii, jj) = row_idx;
			jvec_z(1:n_nz, ii, jj) = col_idx;
			svec_z(1:n_nz, ii, jj) = s;
		end
	end
	
	%% put them all together
	ok_x			= svec_x ~= 0;
	ok_y			= svec_y ~= 0;
	ok_z			= svec_z ~= 0;
	ivec			= [ivec_x(ok_x); ivec_y(ok_y); ivec_z(ok_z)];
	jvec			= [jvec_x(ok_x); jvec_y(ok_y); jvec_z(ok_z)];
	svec			= [svec_x(ok_x); svec_y(ok_y); svec_z(ok_z)];
	G				= sparse(ivec, jvec, svec, 3*Nx*Ny*Nz, Nx*Ny*Nz);
end
