function DEL = laplacian_matrix(x, y, z, Nc, c, order, missing)
	% for a 4D array U which is Nc x Nx x Ny x Nz
	% calculate a discrete laplacian operator
	% DEL which yields
	% del(phi) = DEL * U(:)
	%
	% where phi = U(c, :, :, :)
	% is a scalar field defined on a grid of regular x, y and z
	% as specified by x y and z
	%
	% del(phi) is the flattened version of a 3D array containing d2p/dxidxi
	% at each spatial location x, y, z
	
	%% sizes of stuff
	dx							= x(2) - x(1);
	dy							= y(2) - y(1);
	dz							= z(2) - z(1);
	Nx							= length(x);
	Ny							= length(y);
	Nz							= length(z);
	N_tot						= Nx * Ny * Nz;
	S							= [Nc Nx Ny Nz];
	if nargin < 6
		order					= 2;
	end
	if nargin < 7
		missing	= false(Nx, Ny, Nz);
	end
	
	%% finite difference kernels
	if order == 1
		kernel_x					= [0 1 -2 1 0] / dx^2;
		kernel_y					= [0 1 -2 1 0] / dy^2;
		kernel_z					= [0 1 -2 1 0] / dz^2;
	else
		kernel_x					= [-1 16 -30 16 -1] / (12 * dx^2);
		kernel_y					= [-1 16 -30 16 -1] / (12 * dy^2);
		kernel_z					= [-1 16 -30 16 -1] / (12 * dz^2);
	end
	
	%% d2f/dx^2 operator
	% create entries for calculating all the second order gradients in x
	% row by row of phi
	ivec_x					= zeros(10*Nx, Ny, Nz);
	jvec_x					= zeros(10*Nx, Ny, Nz);
	svec_x					= zeros(10*Nx, Ny, Nz);
	for kk = 1 : Nz
		for jj = 1 : Ny
			missing_row		= missing(:, jj, kk);
			[~,row,col,s]	= gradient_matrix_1d(x, missing_row, kernel_x);
			n_nz				= length(s);
			if n_nz == 0
				continue;
			end
			row_idx			= sub2ind([   Nx Ny Nz],                 row, jj*ones(n_nz,1), kk*ones(n_nz,1));
			col_idx			= sub2ind([Nc Nx Ny Nz], c*ones(n_nz,1), col, jj*ones(n_nz,1), kk*ones(n_nz,1));
			ivec_x(1:n_nz, jj, kk) = row_idx;
			jvec_x(1:n_nz, jj, kk) = col_idx;
			svec_x(1:n_nz, jj, kk) = s;
		end
	end
	ok							= svec_x ~= 0;
	d2fdxx					= sparse(ivec_x(ok), jvec_x(ok), svec_x(ok), Nx*Ny*Nz, Nc*Nx*Ny*Nz);
	
	%% d2f/dy^2 operator
	% create entries for calculating all the second order gradients in y
	% row by row of phi
	ivec_y					= zeros(10*Ny, Nx, Nz);
	jvec_y					= zeros(10*Ny, Nx, Nz);
	svec_y					= zeros(10*Ny, Nx, Nz);
	for kk = 1 : Nz
		for ii = 1 : Nx
			missing_row		= missing(ii, :, kk);
			[~,row,col,s]	= gradient_matrix_1d(y, missing_row, kernel_y);
			n_nz				= length(s);
			if n_nz == 0
				continue;
			end
			row_idx			= sub2ind([   Nx Ny Nz],                 ii*ones(n_nz,1), row, kk*ones(n_nz,1));
			col_idx			= sub2ind([Nc Nx Ny Nz], c*ones(n_nz,1), ii*ones(n_nz,1), col, kk*ones(n_nz,1));
			ivec_y(1:n_nz, ii, kk) = row_idx;
			jvec_y(1:n_nz, ii, kk) = col_idx;
			svec_y(1:n_nz, ii, kk) = s;
		end
	end
	ok							= svec_y ~= 0;
	d2fdyy					= sparse(ivec_y(ok), jvec_y(ok), svec_y(ok), Nx*Ny*Nz, Nc*Nx*Ny*Nz);
	
	%% d2f/dz^2 operator
	% create entries for calculating all the second order gradients in y
	% row by row of phi
	ivec_z					= zeros(10*Nz, Nx, Ny);
	jvec_z					= zeros(10*Nz, Nx, Ny);
	svec_z					= zeros(10*Nz, Nx, Ny);
	for jj = 1 : Ny
		for ii = 1 : Nx
			missing_row		= missing(ii, jj, :);
			[~,row,col,s]	= gradient_matrix_1d(z, missing_row, kernel_z);
			n_nz				= length(s);
			if n_nz == 0
				continue;
			end
			row_idx			= sub2ind([   Nx Ny Nz],                 ii*ones(n_nz,1), jj*ones(n_nz,1), row);
			col_idx			= sub2ind([Nc Nx Ny Nz], c*ones(n_nz,1), ii*ones(n_nz,1), jj*ones(n_nz,1), col);
			ivec_z(1:n_nz, ii, jj) = row_idx;
			jvec_z(1:n_nz, ii, jj) = col_idx;
			svec_z(1:n_nz, ii, jj) = s;
		end
	end
	ok							= svec_z ~= 0;
	d2fdzz					= sparse(ivec_z(ok), jvec_z(ok), svec_z(ok), Nx*Ny*Nz, Nc*Nx*Ny*Nz);	

	%% combine all
	DEL						= d2fdxx + d2fdyy + d2fdzz;
end
	
	