function [L,Lxx,Lyy,Lzz] = oper_del_cfd_o2(xg, yg, zg, ic, jc, kc, c)
	% D = oper_del_cfd_o2(xg, yg, zg, ic, jc, kc)
	% 
	% define a scalar laplacian operator D for the c^th component of the 
	% 3-component, 3D vector field vg (3 x Nx x Ny x Nz) based on second order central
	% finite differences applied at the grid points indexed by (ic,jc,kc)
	%
	% Input parameters:
	%	xg		1 x Nx		x grid
	%	yg		1 x Ny		y grid
	%	zg		1 x Nz		z grid
	%  ic		N x 1			i-index of grid points
	%	jc		N x 1			j-index of grid points
	%  kc		N x 1			k-index of grid points
	%	c		1 x 1			index of component in field to evalute laplacian
	%
	% Outputs:
	%	L		N x 3Nvert		laplacian operator
	%	
	%  for the 3 x Nx x Ny x Nz vector field v 
	%		lap = D * u(:)
	%	gives the scalar laplacian of v(c,:,:,:) at (xg(ic),yg(jc),zg(kc)	
	
	%% sanity check
	Nx			= length(xg);
	Ny			= length(yg);
	Nz			= length(zg);
	N			= numel(ic);
	assert(Nx >= 2);
	assert(Ny >= 2);
	assert(Nz >= 2);
	assert(numel(jc) == N);
	assert(numel(kc) == N);
	Nvert		= Nx * Ny * Nz;
	dx			= xg(2) - xg(1);
	dy			= yg(2) - yg(1);
	dz			= zg(2) - zg(1);
	%assert(all((xg(2:end) - xg(1:end-1)) == dx), 'xg must be uniformly spaced');
	%assert(all((yg(2:end) - yg(1:end-1)) == dy), 'yg must be uniformly spaced');
	%assert(all((zg(2:end) - zg(1:end-1)) == dz), 'zg must be uniformly spaced');
	
	%% get indices of vertices
	ic				= ic(:);
	jc				= jc(:);
	kc				= kc(:);
	I				= ones(N, 1);
	Ng				= [3 Nx Ny Nz];

	% center
	ctr			= sub2ind(Ng, c*I, ic  , jc  , kc  );
	% x neighbours
	xp				= sub2ind(Ng, c*I, ic+1, jc  , kc  );
	xm				= sub2ind(Ng, c*I, ic-1, jc  , kc  );
	% y neighbours
	yp				= sub2ind(Ng, c*I, ic  , jc+1, kc  );
	ym				= sub2ind(Ng, c*I, ic  , jc-1, kc  );
	% z neighbours
	zp				= sub2ind(Ng, c*I, ic  , jc  , kc+1);
	zm				= sub2ind(Ng, c*I, ic  , jc  , kc-1);
	
	%% evaluate contribution to divergence via Stokes' theorem
	row			= (1 : N)';
	Lxx			= sparse([row; row; row], [xp; ctr; xm], [I; -2*I; I]/dx^2, N, 3*Nvert);
	Lyy			= sparse([row; row; row], [yp; ctr; ym], [I; -2*I; I]/dy^2, N, 3*Nvert);
	Lzz			= sparse([row; row; row], [zp; ctr; zm], [I; -2*I; I]/dz^2, N, 3*Nvert);
	L				= Lxx + Lyy + Lzz;
end
