function [D,DuDx,DvDy,DwDz] = oper_div_cfd_o2c(xg, yg, zg, ic, jc, kc)
	% D = oper_div_cfd_o2(xg, yg, zg, ic, jc, kc)
	% 
	% define a divergence operator D for the 3-component, 3D
	% vector field vg (3 x Nx x Ny x Nz) based on second order central
	% finite differences applied at the grid points indexed by (ic,jc,kc)
	%
	% Input parameters:
	%	xg		1 x Nx
	%	yg		1 x Ny
	%	zg		1 x Nz
	%  ic		N x 1
	%	jc		N x 1
	%  kc		N x 1
	%
	% Outputs:
	%	D		N x 3Nvert		divergence operator
	%	
	%  for the 3 x Nx x Ny x Nz vector field v 
	%		div = D * u(:)
	%	gives the divergence at (xg(ic),yg(jc),zg(kc)	
	
	%% sanity check
	Nx			= length(xg);
	Ny			= length(yg);
	Nz			= length(zg);
	N			= numel(ic);
	assert(Nx >= 2);
	assert(Ny >= 2);
	assert(Nz >= 2);
	assert(numel(jc) == N);
	assert(numel(kc) == N);
	Nvert		= Nx * Ny * Nz;
	dx			= xg(2) - xg(1);
	dy			= yg(2) - yg(1);
	dz			= zg(2) - zg(1);
	%assert(all((xg(2:end) - xg(1:end-1)) == dx), 'xg must be uniformly spaced');
	%assert(all((yg(2:end) - yg(1:end-1)) == dy), 'yg must be uniformly spaced');
	%assert(all((zg(2:end) - zg(1:end-1)) == dz), 'zg must be uniformly spaced');
	
	%% get indices of vertices
	ic				= ic(:);
	jc				= jc(:);
	kc				= kc(:);
	I				= ones(N, 1);
	Ng				= [3 Nx Ny Nz];
	
	% u gradient in x
	up				= sub2ind(Ng, 1*I, ic+2, jc  , kc  );
	um				= sub2ind(Ng, 1*I, ic-2, jc  , kc  );
	% v gradient in y
	vp				= sub2ind(Ng, 2*I, ic  , jc+2, kc  );
	vm				= sub2ind(Ng, 2*I, ic  , jc-2, kc  );
	% w gradient in z
	wp				= sub2ind(Ng, 3*I, ic  , jc  , kc+2);
	wm				= sub2ind(Ng, 3*I, ic  , jc  , kc-2);
	
	%% evaluate contribution to divergence
	row			= (1 : N)';
	DuDx			= sparse([row; row], [up; um], [+I; -I]/(4*dx), N, 3*Nvert);
	DvDy			= sparse([row; row], [vp; vm], [+I; -I]/(4*dy), N, 3*Nvert);
	DwDz			= sparse([row; row], [wp; wm], [+I; -I]/(4*dz), N, 3*Nvert);
	D				= DuDx + DvDy + DwDz;
end
