function [D,DuDx,DvDy,DwDz] = oper_div_cfd_o4(xg, yg, zg, ic, jc, kc)
	% D = oper_div_cfd_o2(xg, yg, zg, ic, jc, kc)
	% 
	% define a divergence operator D for the 3-component, 3D
	% vector field vg (3 x Nx x Ny x Nz) based on second order central
	% finite differences applied at the grid points indexed by (ic,jc,kc)
	%
	% Input parameters:
	%	xg		1 x Nx
	%	yg		1 x Ny
	%	zg		1 x Nz
	%  ic		N x 1
	%	jc		N x 1
	%  kc		N x 1
	%
	% Outputs:
	%	D		N x 3Nvert		divergence operator
	%	
	%  for the 3 x Nx x Ny x Nz vector field v 
	%		div = D * u(:)
	%	gives the divergence at (xg(ic),yg(jc),zg(kc)	
	
	%% sanity check
	Nx			= length(xg);
	Ny			= length(yg);
	Nz			= length(zg);
	N			= numel(ic);
	assert(Nx >= 2);
	assert(Ny >= 2);
	assert(Nz >= 2);
	assert(numel(jc) == N);
	assert(numel(kc) == N);
	Nvert		= Nx * Ny * Nz;
	dx			= xg(2) - xg(1);
	dy			= yg(2) - yg(1);
	dz			= zg(2) - zg(1);
	%assert(all((xg(2:end) - xg(1:end-1)) == dx), 'xg must be uniformly spaced');
	%assert(all((yg(2:end) - yg(1:end-1)) == dy), 'yg must be uniformly spaced');
	%assert(all((zg(2:end) - zg(1:end-1)) == dz), 'zg must be uniformly spaced');
	
	%% get indices of vertices
	ic				= ic(:);
	jc				= jc(:);
	kc				= kc(:);
	I				= ones(N, 1);
	Ng				= [3 Nx Ny Nz];
	
	% u gradient in x
	um2			= sub2ind(Ng, 1*I, ic-2, jc  , kc  );
	um1			= sub2ind(Ng, 1*I, ic-1, jc  , kc  );
	up1			= sub2ind(Ng, 1*I, ic+1, jc  , kc  );
	up2			= sub2ind(Ng, 1*I, ic+2, jc  , kc  );
	% v gradient in y
	vm2			= sub2ind(Ng, 2*I, ic  , jc-2, kc  );
	vm1			= sub2ind(Ng, 2*I, ic  , jc-1, kc  );
	vp1			= sub2ind(Ng, 2*I, ic  , jc+1, kc  );
	vp2			= sub2ind(Ng, 2*I, ic  , jc+2, kc  );
	% w gradient in z
	wm2			= sub2ind(Ng, 3*I, ic  , jc  , kc-2);
	wm1			= sub2ind(Ng, 3*I, ic  , jc  , kc-1);
	wp1			= sub2ind(Ng, 3*I, ic  , jc  , kc+1);
	wp2			= sub2ind(Ng, 3*I, ic  , jc  , kc+2);
	
	%% evaluate contribution to divergence
	% gradients are calculated with the stencil
	%	[-1 8 0 -8 1] / (12*dx)
	row			= (1 : N)';
	DuDx			= sparse([row; row; row; row], [up2; up1; um1; um2], [-I; +8*I; -8*I; +I]/(12*dx), N, 3*Nvert);
	DvDy			= sparse([row; row; row; row], [vp2; vp1; vm1; vm2], [-I; +8*I; -8*I; +I]/(12*dy), N, 3*Nvert);
	DwDz			= sparse([row; row; row; row], [wp2; wp1; wm1; wm2], [-I; +8*I; -8*I; +I]/(12*dz), N, 3*Nvert);
	D				= DuDx + DvDy + DwDz;
end
