function [D,DuDx,DvDy,DwDz] = oper_div_fv(xg, yg, zg)
	% D = divop_fg(xg, yg, zg)
	% 
	% define a divergence operator D for the 3-component, 3D
	% vector field vg (3 x Nx x Ny x Nz) based on a finite volume
	% discretisation with trilinear interpolation
	%
	% the vector field v(x) is defined to be piecewise linear
	% as if vg was interpolated with interp3
	%
	% divergence is calculated using a finite volume discretisation using
	% Stokes' theorem
	%
	% result is a Ndiv x Ngrid sparse matrix which yields the flow
	% divergence averaged over each cell
	%
	% Input parameters:
	%	xg		1 x Nx
	%	yg		1 x Ny
	%	zg		1 x Nz
	%
	% Output parameters:
	%	D		Ncell x Nvert
	% where
	%	Ncell = (Nx-1)*(Ny-1)*(Nz-1) = number of cells
	%	Nvert = Nx * Ny * Nz          = number of vertices
	%
	% see divop_fv_derivation.m for derivation of formulae here
	
	%% sanity check
	Nx			= length(xg);
	Ny			= length(yg);
	Nz			= length(zg);
	assert(Nx >= 2);
	assert(Ny >= 2);
	assert(Nz >= 2);
	Ncell		= (Nx-1)*(Ny-1)*(Nz-1);
	Nvert		= Nx * Ny * Nz;
	dx			= xg(2) - xg(1);
	dy			= yg(2) - yg(1);
	dz			= zg(2) - zg(1);
	%assert(all((xg(2:end) - xg(1:end-1)) == dx), 'xg must be uniformly spaced');
	%assert(all((yg(2:end) - yg(1:end-1)) == dy), 'yg must be uniformly spaced');
	%assert(all((zg(2:end) - zg(1:end-1)) == dz), 'zg must be uniformly spaced');
	
	%% get indices of corners of cells
	[ic,jc,kc]	= ndgrid(1:Nx-1, 1:Ny-1, 1:Nz-1);
	ic				= ic(:);
	jc				= jc(:);
	kc				= kc(:);
	I				= ones(Ncell, 1);
	Ng				= [3 Nx Ny Nz];
	
	% u field
	u000			= sub2ind(Ng, 1*I,  ic  , jc  , kc  );
	u001			= sub2ind(Ng, 1*I,  ic  , jc  , kc+1);
	u010			= sub2ind(Ng, 1*I,  ic  , jc+1, kc  );
	u011			= sub2ind(Ng, 1*I,  ic  , jc+1, kc+1);
	u100			= sub2ind(Ng, 1*I,  ic+1, jc  , kc  );
	u101			= sub2ind(Ng, 1*I,  ic+1, jc  , kc+1);
	u110			= sub2ind(Ng, 1*I,  ic+1, jc+1, kc  );
	u111			= sub2ind(Ng, 1*I,  ic+1, jc+1, kc+1);
	% v field
	v000			= sub2ind(Ng, 2*I,  ic  , jc  , kc  );
	v001			= sub2ind(Ng, 2*I,  ic  , jc  , kc+1);
	v010			= sub2ind(Ng, 2*I,  ic  , jc+1, kc  );
	v011			= sub2ind(Ng, 2*I,  ic  , jc+1, kc+1);
	v100			= sub2ind(Ng, 2*I,  ic+1, jc  , kc  );
	v101			= sub2ind(Ng, 2*I,  ic+1, jc  , kc+1);
	v110			= sub2ind(Ng, 2*I,  ic+1, jc+1, kc  );
	v111			= sub2ind(Ng, 2*I,  ic+1, jc+1, kc+1);
	% w field
	w000			= sub2ind(Ng, 3*I,  ic  , jc  , kc  );
	w001			= sub2ind(Ng, 3*I,  ic  , jc  , kc+1);
	w010			= sub2ind(Ng, 3*I,  ic  , jc+1, kc  );
	w011			= sub2ind(Ng, 3*I,  ic  , jc+1, kc+1);
	w100			= sub2ind(Ng, 3*I,  ic+1, jc  , kc  );
	w101			= sub2ind(Ng, 3*I,  ic+1, jc  , kc+1);
	w110			= sub2ind(Ng, 3*I,  ic+1, jc+1, kc  );
	w111			= sub2ind(Ng, 3*I,  ic+1, jc+1, kc+1);
	
	%% evaluate contribution to divergence via Stokes' theorem
	row			= (1 : Ncell)';
	DuDx			= sparse([ row;  row;  row;  row;  row;  row;  row;  row], ...
								[u000; u001; u010; u011; u100; u101; u110; u111], ...
								[  +I;   +I;   +I;   +I;   -I;   -I;   -I;   -I] / (-4*dx), ...
								Ncell, 3*Nvert);
	DvDy			= sparse([ row;  row;  row;  row;  row;  row;  row;  row], ...
								[v000; v001; v010; v011; v100; v101; v110; v111], ...
								[  +I;   +I;   -I;   -I;   +I;   +I;   -I;   -I] / (-4*dy), ...
								Ncell, 3*Nvert);
	DwDz			= sparse([ row;  row;  row;  row;  row;  row;  row;  row], ...
								[w000; w001; w010; w011; w100; w101; w110; w111], ...
								[  +I;   -I;   +I;   -I;   +I;   -I;   +I;   -I] / (-4*dz), ...
								Ncell, 3*Nvert);
	D				= DuDx + DvDy + DwDz;
end
