%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% test_divfree
%
% test the divfree function, by generating an arbitrary, divergence free
% vector field, adding some noise, and removing the non-solenoidal part

%% input parameters

Nx			= 124;
Ny			= 124;
Nz			= 22;
L			= 30;				% size of box
r_core	= 3;				% radius of vortex core
e_vortex = [-0.2050   -0.1241    1.4897];
e_vortex = e_vortex / norm(e_vortex);
Gamma		= 1;
SNR		= 10;
Lambda	= 0.0001;
cmap		= jet;

gradient_operator		= @gradient_o2_coarse;
divfree_operator		= @divfree_o2c;

%% create grid

dx		= 1 / (Nx-1) * L;

x		= (-(Nx-1)/2 : (Nx-1)/2) * dx;
y		= (-(Ny-1)/2 : (Ny-1)/2) * dx;
z		= (-(Nz-1)/2 : (Nz-1)/2) * dx;
[xmat, ymat, zmat] = ndgrid(x, y, z);

%% construct velocity field
% consists of a vortex with its axis in the direction of e_vortex
% passing through origin

fprintf('Generating velocity field ... ');

rmat	= sqrt(xmat.^2 + ymat.^2 + zmat.^2);
r_par	= xmat*e_vortex(1) + ymat*e_vortex(2) + zmat*e_vortex(3);
r_norm= sqrt(rmat.^2 - r_par.^2);

% velocity component tangential to vortex
u_theta	= Gamma * (1 - exp(-r_norm.^2 / r_core^2)) ./ (2 * pi * r_norm);
% velocity component in cartesians
% e_r			vector is e_r = [x,y,z] / r
% e_theta	= cross(e_vortex, e_r)

ux			= zeros([Nx Ny Nz]);
uy			= zeros([Nx Ny Nz]);
uz			= zeros([Nx Ny Nz]);

for kk = 1 : Nz
	for jj = 1 : Ny
		for ii = 1 : Nx
			e_r		= [xmat(ii,jj,kk) ymat(ii,jj,kk) zmat(ii,jj,kk)] / rmat(ii,jj,kk);
			e_theta  = cross(e_r, e_vortex);
			ux(ii, jj, kk) = e_theta(1) * u_theta(ii,jj,kk);
			uy(ii, jj, kk) = e_theta(2) * u_theta(ii,jj,kk);
			uz(ii, jj, kk) = e_theta(3) * u_theta(ii,jj,kk);
		end
	end
end

fprintf('done\n');

%% add noise
rng(0);
u_max					= max([ux(:); uy(:); uz(:)]);
ux_noise				= randn([Nx Ny Nz]) * u_max/SNR;
uy_noise				= randn([Nx Ny Nz]) * u_max/SNR;
uz_noise				= randn([Nx Ny Nz]) * u_max/SNR;
ux_meas				= ux + ux_noise;
uy_meas				= uy + uy_noise;
uz_meas				= uz + uz_noise;

%% filtering

fprintf('Filtering noisy signal ... ');

[ux_filt, uy_filt, uz_filt] = divfree_operator(ux_meas, uy_meas, uz_meas, x, y, z, Lambda);

fprintf('done\n');

%% measure norm of error
err_meas				= norm(ux(:) - ux_meas(:)) ...
						+ norm(uy(:) - uy_meas(:)) ...
						+ norm(uz(:) - uz_meas(:));
err_filt				= norm(ux(:) - ux_filt(:)) ...
						+ norm(uy(:) - uy_filt(:)) ...
						+ norm(uz(:) - uz_filt(:));
fprintf('err before: %f\n', err_meas);
fprintf('err after : %f\n', err_filt);

%% calculate gradients
[dudx, dudy, dudz]	= gradient_operator(ux, dx, dx, dx);
[dvdx, dvdy, dvdz]	= gradient_operator(uy, dx, dx, dx);
[dwdx, dwdy, dwdz]	= gradient_operator(uz, dx, dx, dx);
div_ref					= dudx + dvdy + dwdz;

[dudx, dudy, dudz]	= gradient_operator(ux_meas, dx, dx, dx);
[dvdx, dvdy, dvdz]	= gradient_operator(uy_meas, dx, dx, dx);
[dwdx, dwdy, dwdz]	= gradient_operator(ux_meas, dx, dx, dx);
div_meas					= dudx + dvdy + dwdz;

[dudx, dudy, dudz]	= gradient_operator(ux_filt, dx, dx, dx);
[dvdx, dvdy, dvdz]	= gradient_operator(uy_filt, dx, dx, dx);
[dwdx, dwdy, dwdz]	= gradient_operator(uz_filt, dx, dx, dx);
div_filt					= dudx + dvdy + dwdz;

%% plot velocity field
iz							= 12;
figure(1);
clf;
subplot(1,3,1);
imagesc(ux(:,:,iz));
axis equal tight;

subplot(1,3,2);
imagesc(ux_meas(:,:,iz));
axis equal tight;

subplot(1,3,3);
imagesc(ux_filt(:,:,iz));
axis equal tight;

%% divergence field
figure(2);
clf;
subplot(1,3,1);
imagesc(div_ref(:,:,iz));
axis equal tight;
colorbar;

subplot(1,3,2);
imagesc(div_meas(:,:,iz));
axis equal tight;
colorbar;

subplot(1,3,3);
imagesc(div_filt(:,:,iz));
axis equal tight;
colorbar;