function [uv_ctr, radius] = find_circles(u_axis, v_axis, im, opts)
	% find circles on image using hough transform
	%
	% Input
	%	u_axis	1 x M	u coordinate axis
	%	v_axis	1 x N	v coordinate axis
	%	im		M x N	image to search
	%	opts	struct	options for search
	%
	% Options structure
	%	circle_radius		radius range to search for, pixels
	%	circle_threshold	threshold for identifying edges [0-1]
	%	circle_polarity		polarity of the circular objects, 'bright'/'dark'
	%	circle_binning   	downsample image by this factor to
	%						accelerate search for circles
	%
	% Output
	%	uv_ctr	2 x P	coordinates of circle centres
	%	radius	1 x P	radii of detected circles
	
	%% downsample image
	factor			= opts.circle_binning;
	im_size_full	= size(im);
	assert(all(mod(im_size_full, factor) == [0 0]));
	im_size_down	= im_size_full / factor;
		
	if factor == 1
		% full size, do not downsample
		im_down		= im;
	else
		% coarsen image by binning pixels together 
		im_down		= reshape(im, [factor im_size_down(1) factor im_size_down(2)]);
		im_down		= reshape(mean(mean(im_down, 1), 3), im_size_down);
	end
	
	du				= mean(diff(u_axis)) * factor;
	dv				= mean(diff(v_axis)) * factor;
	u0				= u_axis(1);
	v0				= v_axis(1);
	
	%% find objects in image using hough circle transform
	% scale up search by downsampling factor
	r_search		= round(opts.circle_radius / factor);
	
	% create reference gradient at corner
	% this provides consistency in specifying the threshold for edge pixels
	maxval			= max(im_down(:));
	im_down(1:3,1:3)= (fspecial('sobel')'+fspecial('sobel')) * maxval;
	warning('off', 'images:imfindcircles:warnForSmallRadius');
	[idx_ctr,r_down]= imfindcircles(im_down', r_search, ...
						'ObjectPolarity',	opts.circle_polarity, ...
						'EdgeThreshold',	opts.circle_threshold);
	n_circ			= length(r_down);
	if n_circ == 0
		% provide results with consistent size if no circles found 
		uv_ctr		= zeros(2,0);
		radius		= zeros(1,0);
	else
		% convert indices provided by imfindcircles to image coordinate system
		radius		= r_down' * factor;
		uv_ctr		= diag([du dv]) * (idx_ctr'-1) + [u0; v0];
	end
end
					