function matches = link_nearest_neighbour(re_part_t1, re_part_t2, R_max)
	% identify matches between set of particles re_part_t1 and re_part_t2
	% which are within a specified search radius R_max
	%
	%	Input
	%		3 x M	re_part_t1		set of particle coords at time t1
	%		3 x N	re_part_t2		set of particle coords at time t2
	%		1 x 1	R_max			maximum search radius for match
	%
	%	Output:
	%		K x 2	matches			indices of matching particle pairs
	%								matches(:,1) is index in re_part_t1
	%								matches(:,2) is index in re_part_t2
	%
	
	%% sanity check input
	if isempty(re_part_t1) || isempty(re_part_t2)
		matches = zeros(0,2);
		return;
	end

	assert(size(re_part_t1, 1) == 3);
	assert(size(re_part_t2, 1) == 3);
	n_t1		= size(re_part_t1, 2);
	n_t2		= size(re_part_t2, 2);
	
	if n_t2 == 0 || n_t1 == 0
		matches = zeros(0, 2);
		return;
	end
	
	%% search with nearest neighbour
	[iB,dist]	= knnsearch(re_part_t2.', re_part_t1.', ...
							'K', 1);
	% eliminate matches which are too far apart
	far				= dist > R_max;
	iA				= 1 : n_t1;
	iA(far)			= [];
	iB(far)			= [];
	dist(far)		= [];
	% handle ties
	% there can only be a 1:1 correspondence of A->B
	map				= sparse(iA, iB, 1./dist, n_t1, n_t2);
	[maxv,best]		= max(map,[],1);
	iB				= find(maxv~=0);
	iA				= best(maxv~=0); 

	% assemble in array
	matches			= [iA(:), iB(:)];
end
	