#include "mex.h"
#include <omp.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>

/**** defines ****/
#define CHUNKSIZE							64

#define MIN(a, b)							((a)<(b)?(a):(b))

/**** function declarations ****/
void mexFunction(int, mxArray *[], int, const mxArray *[]);
int minfilt1d(const float *u, float *minu, int Ndims, const mwSize *N, int n, int S);

void mexFunction(	int nlhs,	mxArray *plhs[],
						int nrhs,	const mxArray *prhs[])
{
	/***********
	 * minu = minfilt1d(u, S)
	 *
	 * implements a 1D sliding minimum filter along the nth dimension of u
	 * the van herk algorithm is implemented,
	 * with multithreading to make everything run faster
	 *
	 * minu(i, j) = min(u(k, j)) over i - (S-1)/2 <= k <= i + (S-1)/2
	 *
	 * the window size of the filter is specified by S
	 * S should be an odd number, to preserve symmetry 
	 * if S is not odd, a warning will be issued
	 */

	mxArray *mxminu;
	const mxArray *mxu, *mxS, *mxn;
	const mwSize *N;
	int S, i;
	int n;
	int Ndims;
	const float *u;
	float *minu;
		
	/***************
	 * check right number of arguments have been supplied
	 */
	if(nrhs < 2)
	{
		mexErrMsgTxt("minfilt1d takes a minimum of 2 arguments: minu = minfilt1d(u, S, [n])");
		return;
	}
	if(nrhs > 3)
	{
		mexErrMsgTxt("minfilt1d takes a minimum of 3 arguments: minu = minfilt1d(u, S, [n])");
	}
	if(nlhs != 1)
	{
		mexErrMsgTxt("minfilt1d has 1 output argument: minu = minfilt1d(u, S, [n])");
		return;
	}

	/***************
	 * check arguments have been supplied in the right format
	 */
	mxu			= prhs[0];
	mxS			= prhs[1];
	mxn			= NULL;
	if(nrhs == 3)
		mxn			= prhs[2];

	// if n is specified, then it must be a valid dimension of u
	Ndims			= mxGetNumberOfDimensions(mxu);
	if(mxn)
	{
		if(!mxIsNumeric(mxn) || mxIsComplex(mxn) || mxGetNumberOfElements(mxn) != 1)
		{
			mexErrMsgTxt("n must be a valid dimension of u");
			return;
		}
		n			= (int)mxGetScalar(mxn);
		if(n < 1 || n > Ndims)
		{
			mexErrMsgTxt("n must be a valid dimension of u");
			return;
		}
	}
	else
	{
		n = 1;
	}

	// u must be a real Nd array whose nth dimension is greater than 2 
	// and must be of class single
	N	= mxGetDimensions(mxu);	
	if(Ndims < 2 || !mxIsSingle(mxu) || mxIsComplex(mxu) )
	{
		mexErrMsgTxt("u must be a real, Nd array of class single, whose nth dimension has a size greater than or equal to 3");
		return;
	}
	for(i = 0; i < Ndims; ++i)	
	{
		if((i == (n-1) && N[i] < 3) || N[i] < 1)
		{
			mexErrMsgTxt("u must be a real, Nd array of class single, whose nth dimension has a size greater than or equal to 3");
			return;
		}
	}
	u			= (const float*)mxGetData(mxu);

	// S must be numeric, real, odd, integer and no greater than size(u, 1)
	if(mxGetNumberOfElements(mxS) != 1 || !mxIsNumeric(mxS) || mxIsComplex(mxS))
	{
		mexErrMsgTxt("S must be a real, odd, integer scalar no bigger than the nth dimension of u");
		return;
	}
	S			= (int)mxGetScalar(mxS);
	if(S < 3 || S > N[n-1] || (S%2 != 1))
	{
		mexErrMsgTxt("S must be a real, odd, integer scalar greater than or equal to 3 and no bigger than size(u,1)");
		return;
	}
		
	/*********
	 * allocate memory for output
	 */
	mxminu	= mxCreateNumericArray(Ndims, N, mxSINGLE_CLASS, mxREAL);
	plhs[0]	= mxminu;
	minu		= mxGetData(mxminu);

	
	/*********
	 * pass to minfilt1d to do calculation
	 */
	if(minfilt1d(u, minu, Ndims, N, n, S))
		mexErrMsgTxt("there was an error with the core algorithm of minfilt1d");
	
	return;
}

/****************************************************
 * int minfilt1d(const float *u, float *minu, int Ndims, const mwSize *N, int n, int S)
 *
 * implements the van-herk algorithm for a 1D sliding minimum filter along the nth dimension
 * of u
 *
 * the result is given in minu
 * u is of size N[0]xN[1]x ...
 * the result minu has the same size as u
 *
 * u will be padded with -inf or or +inf values at the edges 
 * to make it fit the windowing kernel
 */

int minfilt1d(const float *u, float *minu, int Ndims, const mwSize *N, int n, int S)
{
	float *g_min, *h_min, *u_pad_min;
	int Npadded, Nsteps, stepsize, Nprimary, Nproduct, numel;
	int i, j, k, offset;

	/***
	 * precalculate how many elements we'll need
	 * for the padded u, and how to iterate over u
	 */
	// Nprimary is the number of elements on the nth dimension
	Nprimary = N[n-1];				
	Npadded	= Nprimary + S - 1;
	Npadded  = (Npadded/S + 1)*S;
	// this loop should mean Nsteps = prod(N) / Nprimary
	// and Nproduct = prod(N) / N(end)
	numel    = 1;
	for(i = 0; i < Ndims; ++i)	{	numel	= numel * N[i];	}
	Nproduct = numel / N[Ndims-1];
	Nsteps   = numel / N[n-1];
	// column major format means adjacent elements in the nth dimension
	// should be stored spaced prod(N(1:(n-1))) apart
	// 
	stepsize = 1;						
	for(i = 0; i < (n-1); ++i)
	{
		stepsize = stepsize * N[i];
	}
	
	/*** enter parallelised section
	 */
	#pragma omp parallel \
		default(none) \
		private(g_min, h_min, u_pad_min, i, j, k, offset) \
		shared(u, minu, Nsteps, stepsize, S, Npadded, Nprimary) \
		num_threads(omp_get_max_threads())
	{
		/* work out how much to pad by, taking into account edge effects
		 * and implementation considerations
		 *
		 * edge effects are handled by adding (S-1)/2 elements
		 * onto either end of u
		 * for a minimum filter, we add (S-1)/2 -inf values on either end
		 *
		 * finally, we need to round up to the next multiple of S
		 * again, padding with -inf or +inf values appropriately
		 */
		g_min		= (float*)malloc(Npadded*sizeof(float));
		h_min		= (float*)malloc(Npadded*sizeof(float));
		u_pad_min= (float*)malloc(Npadded*sizeof(float));
		for(i = 0; i < Npadded; ++i)	{	u_pad_min[i] = FLT_MAX;		}

		// iterate minimum operation over all the remaining dimensions of u
		#pragma omp for schedule(static, CHUNKSIZE) nowait
		for(j = 0; j<Nsteps; ++j)
		{
			/* apply the minimum operation over the nth dimension of u
			 */
			offset = (j%stepsize) + (j / stepsize) * (stepsize * Nprimary);

			// copy into padded vector along the nth dimension of u
			// adjacent elements over the nth dimension of u are spaced stepsize apart,
			// 
			for(i=0; i<Nprimary; ++i)
			{
				/*if((offset + i*stepsize) >= numel)
					return -1;*/
				u_pad_min[(S-1)/2 + i] = u[offset + i*stepsize];
			}

			// calculate g and h
			for(i=0; i < Npadded; i += S)
			{
				// first operation:
				//		g[x] = u[x] for x = 0, S, 2S ...
				//		h[x] = u[x] for x = S-1, 2S-1, ...
				g_min[i]    = u_pad_min[i];
				h_min[i+S-1]= u_pad_min[i+S-1];

				// second operation: 
				//    g[x] = min(g[x-1], u[x]) for x = 1 .. S-1
				//    h[x] = min(h[x+1], u[x]) for x = S-1,
				for(k = 1; k < S; ++k)
				{
					g_min[i+k]    = MIN(g_min[i+k-1], u_pad_min[i+k    ]);
					h_min[i+S-1-k]= MIN(h_min[i+S-k], u_pad_min[i+S-1-k]);
				}
			}

			// calculate result from g and h
			// result[x] = min(g[x+(S-1)/2], h[x-(S-1)/2])
			// but since elements are padded, x = 0 corresponds to g_min[(S-1)/2]
			// so
			// result[x] = min(g[x+(S-1)], h[x])
			for(i = 0; i < Nprimary; ++i)
			{
				minu[offset + i*stepsize] = MIN(g_min[i+S-1], h_min[i]);
			}
		}

		// free the memory we allocated earlier
		free(g_min);
		free(h_min);
		free(u_pad_min);
	} /* end of parallelised section */

	return 0;
}
