function W = trilinear_coeffs(x_grid, y_grid, z_grid, gl_pos)
	% W = trilinear_coeffs(x_grid, y_grid, z_grid, gl_pos)
	% 
	% obtain the set of trilinear coefficients W such that
	% fi = W * f(:)
	% is the trilinear interpolation of f(x,y,z) at points gl_pos
	%
	% Input:
	%	x_grid	1 x Nx x axis of regular cartesian grid upon which f(x,y,z)
	%				is defined
	%	y_grid	1 x Ny
	%	z_grid	1 x Nz
	%	gl_pos	3 x N array of coordinates at which to interpolate f(x,y,z)
	%
	% Output:
	%	W			N x (Nx*Ny*Nz) sparse array of weights for trilinear
	%				interpolation
	%
	
	%% get size of stuff
	Nx				= length(x_grid);
	Ny				= length(y_grid);
	Nz				= length(z_grid);
	N				= size(gl_pos,2);
	n_grid		= Nx * Ny * Nz;
	
	%% find nearest grid corners
	% gl_pos(:,n) will lie on grid between
	% x_grid(i0(n)) <= gl_pos(1,n) < x_grid(i0(n)+1)
	% y_grid(j0(n)) <= gl_pos(2,n) < y_grid(j0(n)+1)
	% z_grid(k0(n)) <= gl_pos(3,n) < z_grid(k0(n)+1)
	[~,i0]		= histc(gl_pos(1,:), x_grid);
	[~,j0]		= histc(gl_pos(2,:), y_grid);
	[~,k0]		= histc(gl_pos(3,:), z_grid);
	
	%% reject extrapolated points
	% we are unable to extrapolate, so we ignore points which
	% are outside the grid
	b_ok			= i0 >= 1 & i0 < Nx ...
					& j0 >= 1 & j0 < Ny ...
					& k0 >= 1 & k0 < Nz;
	n_ok			= sum(b_ok);
	gl_pos		= gl_pos(:, b_ok);
	i0				= i0(b_ok);
	j0				= j0(b_ok);
	k0				= k0(b_ok);
	i_ok			= find(b_ok);

	%% find linear indices of corners
	ijk000		= sub2ind([Nx Ny Nz], i0  , j0  , k0  );
	ijk001		= sub2ind([Nx Ny Nz], i0  , j0  , k0+1);
	ijk010		= sub2ind([Nx Ny Nz], i0  , j0+1, k0  );
	ijk011		= sub2ind([Nx Ny Nz], i0  , j0+1, k0+1);
	ijk100		= sub2ind([Nx Ny Nz], i0+1, j0  , k0  );
	ijk101		= sub2ind([Nx Ny Nz], i0+1, j0  , k0+1);
	ijk110		= sub2ind([Nx Ny Nz], i0+1, j0+1, k0  );
	ijk111		= sub2ind([Nx Ny Nz], i0+1, j0+1, k0+1);
	
	%% width of grid cells
	dx_grid		= x_grid(2:end) - x_grid(1:end-1);
	dy_grid		= y_grid(2:end) - y_grid(1:end-1);
	dz_grid		= z_grid(2:end) - z_grid(1:end-1);
	
	%% calculate the fractional component of x
	% normalised to a grid of unit spacing
	xp				= (gl_pos(1,:) - x_grid(i0)) ./ dx_grid(i0);
	yp				= (gl_pos(2,:) - y_grid(j0)) ./ dy_grid(j0);
	zp				= (gl_pos(3,:) - z_grid(k0)) ./ dz_grid(k0);
	
	%% calculate the weighting of each corner
	% the trilinear interpolation of f(x,y,z) over the domain
	% [0,1], [0,1], [0,1] is given by
	% F(x,y,z) = f000*(1 - x - y - z + x*y + x*z + y*z - x*y*z)
	%			  + f001*(            z       - x*z - y*z + x*y*z)
	%			  + f010*(        y     - x*y       - y*z + x*y*z)
	%			  + f011*(                            y*z - x*y*z)
	%			  + f100*(    x         - x*y - x*z       + x*y*z)
	%			  + f101*(                      x*z       - x*y*z)
	%			  + f110*(                x*y             - x*y*z)
	%			  + f111*(                                + x*y*z)
	C000			= 1 - xp - yp - zp + xp.*yp + xp.*zp + yp.*zp - xp.*yp.*zp;
	C001			=               zp          - xp.*zp - yp.*zp + xp.*yp.*zp;
	C010			=          yp      - xp.*yp          - yp.*zp + xp.*yp.*zp;
	C011			=                                    + yp.*zp - xp.*yp.*zp;
	C100        =     xp           - xp.*yp - xp.*zp          + xp.*yp.*zp;
	C101			=                             xp.*zp          - xp.*yp.*zp;
	C110			=                    xp.*yp                   - xp.*yp.*zp;
	C111			=                                             + xp.*yp.*zp;
	
	%% create sparse matrix
	%{
	W000			= sparse(i_ok, ijk000, C000, N, n_grid);
	W001			= sparse(i_ok, ijk001, C001, N, n_grid);
	W010			= sparse(i_ok, ijk010, C010, N, n_grid);
	W011			= sparse(i_ok, ijk011, C011, N, n_grid);
	W100			= sparse(i_ok, ijk100, C100, N, n_grid);
	W101			= sparse(i_ok, ijk101, C101, N, n_grid);
	W110			= sparse(i_ok, ijk110, C110, N, n_grid);
	W111			= sparse(i_ok, ijk111, C111, N, n_grid);
	W				= W000 + W001 + W010 + W011 + W100 + W101 + W110 + W111;
	%}
	
	W				= sparse([ i_ok(:);  i_ok(:);  i_ok(:);  i_ok(:);  i_ok(:);  i_ok(:);  i_ok(:);  i_ok(:)], ...
								[ijk000.'; ijk001.'; ijk010.'; ijk011.'; ijk100.'; ijk101.'; ijk110.'; ijk111.'], ...
								[  C000.';   C001.';   C010.';   C011.';   C100.';   C101.';   C110.';   C111.'], ...
								N, n_grid);
	
end
