#include "parse_input.h"
#include "common.h"

/****************************************************
 * const float *parse_single_vector(mxArray *pmxArray, int Nmin, int Nmax, int *N)
 * 
 * parses a matlab vector specified by pmxArray into a C floating point array
 * if the dimension of the vector is less than Nmin or greater than Nmax, 
 * the function will return null
 * the number of elements in the vector is saved in N
 */

const float *parse_single_vector(const mxArray *pmxArray, int Nmin, int Nmax, int *N)
{
	int S[2];
	const float *fRet;

	/* check it's a vector */
	if(mxGetNumberOfDimensions(pmxArray) != 2) { return NULL; }
	/* get dimensions and check they're within bounds */
	S[0] = (int)mxGetM(pmxArray);
	S[1] = (int)mxGetN(pmxArray);
	if(S[0] != 1 && S[1] != 1) { return NULL; } // not a vector
	if(S[0] == 0 || S[1] == 0) { return NULL; } // has no elements
	S[0] = MAX(S[0], S[1]);
	if(S[0] < Nmin || (S[0]>Nmax && Nmax > 0)) { return NULL; }
	/* check it's the right type */
	if(!mxIsClass(pmxArray, "single") || mxIsComplex(pmxArray))
	{ return NULL; }
	/* get data */
	fRet = (const float*)mxGetData(pmxArray);
	if(N) { *N = S[0]; }

	return fRet;
}

/****************************************************
 * const float *parse_single_array(mxArray *pmxArray, int Ndims, int *Nmin, int *Nmax, int *N)
 * 
 * parses a matlab array specified by pmxArray into a C floating point array
 * if the dimension of the vector is less than Nmin or greater than Nmax, 
 * the function will return null
 * the size of the array is saved in N
 */

const float *parse_single_array(const mxArray *pmxArray, int Ndims, int *Nmin, int *Nmax, int *N)
{
	const mwSize *mwS;
	const float *fRet;
	int i;

	/* check it has the right number of dimensions */
	if(mxGetNumberOfDimensions(pmxArray) != Ndims) { return NULL; }
	/* get dimensions and check they're within bounds */
	mwS	= mxGetDimensions(pmxArray);
	for(i = 0; i < Ndims; ++i)
	{
		if(Nmin && mwS[i] < Nmin[i])
			return NULL;
		if(Nmax && mwS[i] > Nmax[i] && Nmax[i] > 0)
			return NULL;
	}
	/* check it's the right type */
	if(!mxIsClass(pmxArray, "single") || mxIsComplex(pmxArray))
	{ return NULL; }
	/* get data */
	fRet = (const float*)mxGetData(pmxArray);
	if(N)
	{
		for(i = 0; i < Ndims; ++i)
		{
			N[i] = mwS[i];
		}
	}

	return fRet;
}