function [gl_pos, gl_dist, ij_dist] = triangulate(camera, ij_pos, b_dewarp)
	% [gl_pos, gl_dist, ij_dist] = triangulate(camera, ij_pos)
	%
	% triangulate the location of points in object space
	% given their projections in image space in N cameras
	%
	% input:
	%      camera		structure containing calibration information
	%						for each of N cameras
	%      ij_pos		2 x M x N array of positions of points in the image
	%						space of each of N cameras
	%		 b_dewarp	flag; if true, apply dewarping
	% 
	% output:
	%		gl_pos		3 x M array of positions of points in object space
	%						which minimises the sum of squared distances of
	%                 each point p to its back projection along line l
	%		gl_dist		M x N array of distances between points and the
	%						back-projected rays from each camera
	%		ij_dist		M x N array of distances between points and their
	%						projection on each camera
	%						
	%
	% for more details, see: 
	% Mann 1999: Experimental Study of Relative, Turbulent Diffusion
	% pages 26-27
	
	%% sanity checking
	n_cameras		= length(camera);
	n_points		= size(ij_pos, 2);
	dtype			= class(ij_pos);
	if n_points < 1
		gl_pos		= zeros(3, 0, n_cameras, dtype);
		gl_dist		= zeros(0, n_cameras, dtype);
		ij_dist		= zeros(0, n_cameras, dtype);
		return;
	end
	
	if size(ij_pos, 3) ~= n_cameras
		error('size(ij_pos,3) must match number of cameras');
	end
	if size(ij_pos, 1) ~= 2
		error('size(ij_pos,1) must be 2');
	end
	if nargin < 3
		b_dewarp		= false;
	end
	
	%% undo radial distortion
	ij_dewarped				= ij_pos;
	if b_dewarp
		for c = 1 : n_cameras
			ic					= camera(c).calib.ic;
			kr					= camera(c).calib.kr;
			ij_dewarped(:,:,c)	= inverse_radial_distortion(ij_pos(:,:,c), ic, kr);
		end
	end
	
	%% calculate ray vectors
	% these are unit vectors which correspond to the direction of rays
	% back-projected from the location of each point on each camera
	warning('off', 'MATLAB:nearlySingularMatrix')
	b_mat			= zeros(3, n_points, n_cameras, dtype);
	xc_mat			= zeros(3, 1, 1, n_cameras, dtype);
	for c = 1 : n_cameras
		% unit vector
		G				= camera(c).calib.G;
		R				= camera(c).calib.R;
		GR				= G*R;
		b				= GR \ [ij_dewarped(:,:,c); ones(1,n_points)];
		b_norm			= sqrt(sum(b.^2, 1));
		b_mat(:,:,c)	= bsxfun(@rdivide, b, b_norm);
		xc_mat(:,1,1,c)	= camera(c).calib.xc;
	end
	warning('on', 'MATLAB:nearlySingularMatrix')
	xc_mat			= repmat(xc_mat, [1 1 n_points 1]);
	
	%% find projection tensor
	% Pb_{ij} = d_{ij} - b_i b_j
	b_outer			= bsxfun(@times,	reshape(b_mat, [3 1 n_points n_cameras]), ...
										reshape(b_mat, [1 3 n_points n_cameras]));
	Pb_mat			= bsxfun(@minus, eye(3), b_outer);
	Pb_sum			= sum(Pb_mat, 4);
	
	%% project origin
	%Pb_x0_mat		= mmx('mult', Pb_mat, xc_mat);
	Pb_x0_mat		= pagemtimes(Pb_mat, xc_mat);
	Pb_x0_sum		= sum(Pb_x0_mat, 4);
	
	%% solve for 3D position
	gl_pos			= pagemldivide(Pb_sum, Pb_x0_sum);
	%gl_pos			= mmx('backslash', Pb_sum, Pb_x0_sum);
	gl_pos			= squeeze(gl_pos);
	
	%% calculate ray-point distance in object space
	if nargout > 1
		gl_dist		= zeros(n_points, n_cameras);
	end
	
	%% calculate distance to projection in image space
	if nargout > 2
		% project 
		ij_proj		= zeros(2, n_points, n_cameras);
		for c = 1 : n_cameras
			ij_proj(:,:,c)		= pinhole_model(gl_pos, camera(c).calib, b_dewarp);
		end
		% calculate residual
		ij_res		= ij_proj - ij_pos;
		ij_dist		= permute(sqrt(sum(ij_res.^2, 1)), [2 3 1]);
	end
end