function b_outlier = validate(ipr)
	% b_outlier = validate(ipr)
	%
	% validate velocity vectors for outliers
	% Output:
	% b_outlier, a 4 x n_part array of boolean flags with the following rows
	%
	% b_outlier(1,:)	signal to noise ratio is below threshold
	% b_outlier(2,:)	particle is not bright enough
	% b_outlier(3,:)	particle velocity outside velocity limits
	% b_outlier(4,:)	particle velocity is outlier with respect to neighbours

	%% low snr
	b_low_snr		= ipr.pimg_snr < ipr.opts.snr_min_db;
	%% too dim
	b_too_dim		= ipr.d_part < sqrt(ipr.opts.I_min);
	assert(all(ipr.d_part >= 0), 'found negative diameter');
	%% too fast
	b_vel_lim		= any(ipr.dx_part < ipr.opts.dx_lim(:,1), 1) ...
					| any(ipr.dx_part > ipr.opts.dx_lim(:,2), 1);
	
	%% large residual with respect to neigbours
	% outlier detection using residual with respect to nearest neighbours
	% based on Schanz et. al (2016) Section 2.4
	% from Schanz. et al:
	% "The 10–50 closest ones are chosen, their velocities averaged (vavg), and
	% the root mean square of the velocities difference in relation
	% to vavg is computed (rmsv), giving a coarse measure of the
	% velocity gradients present. The velocity difference v of the
	% current particle to vavg is determined, and if v > To · rmsv ,
	% the particle is deleted. The outlier threshold factor To can
	% be chosen according to the flow and the spatial sampling of
	% the scales. Typical values are 5–15. If not enough particles
	% are found in the vicinity, the particle is left as is."
	
	n_part			= ipr.count();
	[i_nn,d_nn]		= knnsearch(ipr.re_part.', ipr.re_part.', 'K', ipr.opts.outlier_nn+1);
	% velocity of this particle and its nearest neighbours
	v_nn			= reshape(ipr.dx_part(:, i_nn), [3 n_part ipr.opts.outlier_nn+1]);
	% exclude far particles from calculation
	b_far			= d_nn > ipr.opts.outlier_radius;
	v_nn(:,b_far)	= nan;
	n_neighbours	= sum(~b_far, 2).';
	% mean velocity of the nearest neighbours
	v_mean_nn		= mean(v_nn(:,:,2:end), 3, 'omitnan');
	% velocity difference to mean of neighbours
	delta_v			= v_nn - v_mean_nn;
	delta_v_norm	= sum(delta_v.^2,1);
	% mean square difference of neighbours
	delta_v_ms		= mean(delta_v_norm(:,:,2:end),3,'omitnan');
	% check difference
	b_vel_nn		= delta_v_norm(1,:,1) > ipr.opts.outlier_sigma^2 * delta_v_ms ...
					& n_neighbours > ipr.opts.outlier_minnn;
	
	%% combine conditions together
	b_outlier		= cat(1, b_low_snr, b_too_dim, b_vel_lim, b_vel_nn);
end
	