function [re_part, D_part, im_part, img_proj_ary] = rgbipr_reconstruct(setup, opts, img_ary, fg)
	% [re_ipr, D_ipr, im_ipr] = rgbipr_reconstruct(camera, otf, opts, img, fg)
    %
	% apply RGB Iterative Particle Reconstruction to reconstruct
	% a particle field, given a first guess of the particle field
	%
	% Input:
	%	setup       setup structure containing calibration details
    %
	%	opts        structure with options for reconstruction
    %
	%	img_ary		set of images to perform reconstruction of
    %               1 x n_c cell array of images
    %
    %   fg          first guess structure, containing fields
    %       re_part 3 x n_p     positions of particles at t-0
    %       v_part  3 x n_p     velocities of particles at t=0
    %       A_part  1 x n_p     amplitudes (brightness) of particles
	%
	% Output:
    %   ipr         iterative particle reconstruction structure containing
    %      
    %   re_part     3 x n_p     positions of particles at t=0
    %   v_part      3 x n_p     velocities of particles at t=0
    %   A_part      1 x n_p     amplitudes (brightness) of particles
	%   img_recon   1 x n_c     cell array of reconstructed images
    
    %% load variables from calibration
	n_cameras			= setup.n_cameras;
	camera				= setup.camera;
	calib				= [camera.calib];
    otf                 = setup.otf;
    delta_t             = setup.delta_t;

	% image space coordinate system
	i_axis				= camera(1).i_axis;
	j_axis				= camera(1).j_axis;
	i_lim				= sort(i_axis([1 end]));
	j_lim				= sort(j_axis([1 end]));
	[imat, jmat]		= ndgrid(i_axis, j_axis);
	n_i					= length(i_axis);
	n_j					= length(j_axis);
	bayer				= camera(1).bayer;
	delta_t				= setup.delta_t;
	t					= [-1 0 1]*delta_t;
	
	% recover triangulation and matching options structures
	mopts				= opts.matching;
	% add limits to reconstructino
	mopts.re_lim		= setup.re_bb_lim;
	% add pre-built adjacency matrix
	mopts.adjacency		= setup.adjacency;
	% add time axis for tracking
	mopts.t				= t;
    mopts.indent		='        ';
	mopts.b_verbose     = opts.b_verbose;
    
	%% first guess
    n_part              = size(fg.re_part, 2);
    assert(size(fg.re_part, 1) == 3);
    assert(size(fg.v_part, 1) == 3);
    assert(size(fg.v_part, 2) == n_part);
    % displacement
    ipr.dx_part         = ipr.v_part * delta_t;
    % evaluate brightness
    ipr.A_part          = zeros(1, n_part);
    [~,pimg]            = rgbipr_project(camera, otf, ipr);
    error('todo: implement finding particle brightness');
    % define diameter
    ipr.d_part          = sqrt(ipr.A_part);
    
	%% IPR options
	n_iter			= opts.n_ipr;
	b_verbose		= opts.b_verbose;
	
	%% start off with first guess
    ipr             = fg;
    
	%% iterate
	for it = 1 : n_iter
		%% shake particles
		if b_verbose
			fprintf('        shaking...\n');
		end
		[ipr, img_proj, img_res_norm, img_snr] = ...
			rgbipr_shake(camera, otf, img_ary, ipr, opts);
		
		%% eliminate bad matches
		% retain only those particles where the signal to noise ratio is
		% better than all the alternatives, and if it is above a minimum
		% threshold
		
		% bad fit condition
		b_accept_fit		= ipr.img_snr > opts.snr_min_db;
        % too dim
        b_accept_bright     = ipr.d_part > sqrt(opts.I_min);
        b_accept            = b_accept_fit & b_accept_bright;
        ipr                 = rgbipr_select(ipr, b_accept);
        if b_verbose
			n_bad           = nnz(~b_accept);
			fprintf('        eliminated %5d bad fit / dim particles\n', n_bad);
		end

        %% reproject particles and calculate residual image
		% here, we take our current guess of the particle field
		% and produce a reconstruction of the scanning images
		if b_verbose
			fprintf('it=%2d:  reprojecting %d particles ...\n', it, n_part);
        end
        % reproject and measure residual
        img_proj_ary    = rgbipr_project(camera, otf, ipr);
        img_res_ary     = cell(1, n_cameras);
        for ic = 1 : n_cameras
            img_res_ary{ic}	= img_ary{ic} - img_proj_ary{ic};
        end
        
        %% retriangulate
        % identify particles from residual images in frame 2
        % then estimate a particle velocity
        %   possibly from predictor field? scattered interpolant
        frames          = rgbipr_retriangulate(camera, img_res_ary, mopts);
        re_part_new     = frames{2}.re_part;
        n_new           = size(re_part_new, 2);
        interpolator    = scatteredInterpolant(ipr.re_part.', ipr.dx_part(1,:).', 'linear');
        dx_part_new     = zeros(3, n_new);
        dx_part_new(1,:)= interpolator(re_part_new.');
        for c = 2 : 3
            interpolator.Values=ipr.dx_part(c,:).';
            dx_part_new(c,:)= interpolator(re_part_new.');
        end
        
		%% identify and triangulate particles in residual images
        %{
		n_part_before_retri		= n_part;
		if b_verbose
			fprintf('        retriangulating on residual image...\n');
		end
		[re_retri, im_retri, amp_retri] = sipr_retriangulate(sheet, camera, img_res_ary, mopts);
		n_retri			= size(re_retri, 2);
		if n_retri > 0
			%% get first guess of brightness brightness
			% we do a least-squares fit to get the amplitude of the particle
			% our Ansatz is that we have a set of observations g_i which 
			% we model as
			%		g_i^ = D^2 * f_i
			%
			% g_i is part of our residual image 
			% f_i is our first guess of the point spread function for this particle
			%
			% the solution for the optimal D^2 in the least square sense is
			%		D^2 = sum_i(f_i g_i) / sum_i(f_i f_i)
			%
			% only non-negative amplitudes are acceptable, so we add a minimum
			% threshold of zero
			fg_sum			= zeros(1, n_retri);
			ff_sum			= zeros(1, n_retri);
			D2_est			= zeros(n_cameras, n_retri);
			
			for ic = 1 : n_cameras
				[~,pimg_retri,pimg_retri_map] = sipr_project(sheet, camera(ic), otf(ic), re_retri, ones(1, n_retri));
				% look up residual image
				map_row		= pimg_retri_map(:,:,1);
				map_col		= pimg_retri_map(:,:,2);
				map_dep		= pimg_retri_map(:,:,3);
				map_ok		= map_row>=1 & map_row <= Ni & map_col >=1 & map_col <= Nj;
				row_ok		= map_row(map_ok);
				col_ok		= map_col(map_ok);
				dep_ok		= map_dep(map_ok);
				n_observable= size(pimg_retri, 1);
				rimg_retri	= zeros([n_observable n_retri]);
				rimg_retri(map_ok) = img_res_ary{ic}(sub2ind([Ni Nj n_sheets], row_ok, col_ok, dep_ok));
				% add to variance and covariance sum
				fg				= sum(rimg_retri .* pimg_retri, 1);
				ff				= sum(pimg_retri .* pimg_retri, 1);
				fg_sum		= fg_sum + fg;
				ff_sum		= ff_sum + ff;
				% per-camera guess of brightness
				D2_est(ic,:) = max(0, fg ./ ff);
			end
			% MART-style guess of amplitude 
			%D2_both			= D2_est;
			D2_est			= sqrt(D2_est(1,:) .* D2_est(2,:));
			
			% pure sum-of-squares type guess of amplitude
			%D2_est			= fg_sum ./ ff_sum;
			
			D_retri			= sqrt(max(D2_est,0));
			
			%% particles should be bright enough
			ok					= D_retri(1,:) > sqrt(opts.I_min);
			re_retri			= re_retri(:,ok);
			D_retri			= D_retri(:,ok);
			n_retri			= nnz(ok);
			% update model
			if n_retri > 0
				re_part		= cat(2, re_part, re_retri);
				D_part		= cat(2,  D_part,  D_retri);
				n_part		= n_part + n_retri;
			end
			if b_verbose
				fprintf('        added %d particles...\n', n_retri);
			end
		end
		
		%% merge particles
		n_part_before_merge	= n_part;
		[re_part,~,idx]		= sipr_remove(re_part, D_part, 1:n_part, opts.R_nn_obj);
		D_part					= D_part(:,idx);
		n_part					= size(re_part, 2);
		if b_verbose
			fprintf('        eliminated %d redundant particles\n', n_part_before_merge-n_part);
		end
		%}
	end
	
	%% reproject particles one last time
	img_proj_ary	= cell(1, n_cameras);
	im_part		= zeros(3, n_part, n_cameras);
	for ic = 1 : n_cameras
		[im_part(:,:,ic),~,~,img_proj_ary{ic}] = ...
			sipr_project(sheet, camera(ic), otf(ic), re_part, D_part(1,:));
	end
end