function [ipr0, pimg_ary_after, pimg_res_ary, pimg_snr_db] = rgbipr_shake(setup, img_ary, ipr0, opts)
	% [re_part,d_part] = rgbipr_shake(camera, otf, img_ary, ipr, opts)
	% 
	% Apply shake step in shake the box
	% Based on an initial guess of the particle distribution, iteratively
	% refine particle position and intenisty to match the image
	%
	% The Levenberg-Marquardt algorithm is implemented to iteratively solve
	% for the optimum particle distribution. 
	%
    % Input:
    %   camera      1 x n_c     camera model structure
    %   otf         1 x n_c     otf model structure
    %   img_ary     1 x n_c     cell array of particle images
    %
    %   ipr, reconstruction structure containing following fields
    %   re_part     3 x n_p     particle object space positions
    %   dx_part     3 x n_p     particle displacement relative to frame 2
    %   A_part      1 x n_p     particle brightness
    %
    %   opts        IPR options structure containing the following fields
    %   dx_max      maximum displacement (3 x 1)
    %   

	%% load calibration
    n_cameras		= setup.n_cameras;
	camera			= setup.camera;
    otf             = setup.otf;
    re_lim          = setup.re_lim;
    re_fov          = abs(re_lim(2,:) - re_lim(1,:));
    
    %% sanity check
	n_iter			= opts.n_shake;
	assert(length(otf) == n_cameras);
	n_part			= size(ipr0.re_part,2);
	assert(size(ipr0.re_part,1)==3);
		
	%% support
	support			= otf(1).support;
	S				= size(support);
	n_support_px	= S(1) * S(2);
    
    %% define sorting plane
    % normal distance from this plane defines ordering of particles
    % we define this as the longest dimension of the reconstruction volume
    [~,i_long]      = max(re_fov);
    e_sort          = zeros(3,1);
    e_sort(i_long)  = 1;
	
	%% image space support
	% to increase sparsity, we only explicitly optimise the fit of image
	% values which are non-zero. This avoids optimising on regions with no
	% particles
	img_mask		= cell(1,n_cameras);
	for ic = 1 : n_cameras
		img_mask{ic}	= img_ary{ic} > 0;
	end
	
	%% refine position over n_iter iterations
	ipr0.lambda	    = opts.lambda0 * ones(1,n_part);
    % set bounds on displacement of particles during shaking procedure
	ipr0.re_part_lb	= bsxfun(@minus, ipr0.re_part, opts.dx_max_step(:));
	ipr0.re_part_ub	= bsxfun(@plus , ipr0.re_part, opts.dx_max_step(:));
		
	for n = 1 : n_iter
		%% sort particles to optimise sparsity structure
		% particles which are likely to interact with each other will tend
		% to lie on the same epipolar plane. by taking the dot-product
		% of position with the sorting vector, we measure the height of each 
		% particle from the reference plane
		% then, by re-sorting according to this height, we place particles
		% that are likely to interact with each other at similar indices 
		% in the Jacobian matrix. this results in a banded structure for
		% J'J, which is better suited for solution with a sparse solver.
		
		% sort by height
		re_sort					= e_sort' * ipr0.re_part;
		[re_sort, reidx]		= sort(re_sort, 'ascend');
        ipr0                    = rgbipr_select(ipr0, reidx);
		
        %% project particles onto images
        [img_proj_it0, pimg0]   = rgbipr_project(camera, otf, ipr0);
        % evaluate residual
        img_res_it0				= cell(1,n_cameras);
        for rgb = 1 : 3
            img_res_it0{rgb}    = img_ary{rgb} - img_proj_it0{rgb};
        end
		
		%% iterate to obtain jacobian, gramian and residual
        % jacobian J(ii,jj) represents change in projected image with respect to
        % the model variables re_part, dx_part and A_part
        % 
        % it is of dimension (n_px x 7*n_part)
        %
        % each particle has 7 columns in the Jacobian matrix
        % model parameters are plaid like
        % [D1 x1 y1 z1 dx1 dy1 dz1, D2 x2 y2 z2 dx2 dy2 dz2, ...]
        % 
		
        if opts.b_verbose
			fprintf('shake%02d projecting and measuring residual\n', n);
        end
		
        % Gramian moment matrix
		JtJ						= sparse([], [], [], 7*n_part, 7*n_part);
        % residual
		JtDy					= zeros(7*n_part,1);
        % TODO: check whether it's OK to accumulate J'J like this for rgb images 
		
		for ic = 1 : n_cameras
			%% size of this image
			n_i					= size(img_ary{ic},1);
			n_j					= size(img_ary{ic},2);
            n_px                = n_i*n_j;
            
            %% accumulate jacobian over time
            big_jac             = sparse(7*n_part, 7*n_part);
            for rgb = 1 : 3
                %% map jacobian back to particle images
                % indices of particles 
                ind                 = pimg0(rgb,ic).ind;
                % row and column coordinates of particle images
                row_map				= pimg0(rgb,ic).map(:,:,1);
                col_map				= pimg0(rgb,ic).map(:,:,2);
                % take care of out of bounds elements in map
                % dep_map is always in bounds by construction
                map_ok				= row_map >= 1 & row_map <= n_i ...
                                    & col_map >= 1 & col_map <= n_j;
                % create map of indices where map is valid
                ind_map				= zeros([n_support_px n_part]);
                ind_map(map_ok)     = sub2ind([n_i n_j], row_map(map_ok), col_map(map_ok));

                %% make big-ass Jacobian matrix
                jac_row_idx			= ind_map;
                jac_col_idx			= repelem(ind, n_support_px);
                n_ok				= nnz(map_ok);
                pimg_jac_x			= pimg0(rgb,ic).jac(:,:,1);
                pimg_jac_y			= pimg0(rgb,ic).jac(:,:,2);
                pimg_jac_z			= pimg0(rgb,ic).jac(:,:,3);
                pimg_jac_D			= pimg0(rgb,ic).img;
                
                % jacobian with respect to particle brightness
                big_jac_D           = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+1, pimg_jac_D(map_ok), n_px, n_part*7, n_ok); 
                % jacobian with respect to particle position at time t0
                big_jac_x           = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+2, pimg_jac_x(map_ok), n_px, n_part*7, n_ok);
                big_jac_y           = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+3, pimg_jac_y(map_ok), n_px, n_part*7, n_ok);
                big_jac_z           = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+4, pimg_jac_z(map_ok), n_px, n_part*7, n_ok);
                % jacobian with respect to 
                if rgb == 1 || rgb == 3
                    big_jac_dx      = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+5, (rgb-2)*pimg_jac_x(map_ok), n_px, n_part*7, n_ok);
                    big_jac_dy      = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+5, (rgb-2)*pimg_jac_y(map_ok), n_px, n_part*7, n_ok);
                    big_jac_dz      = sparse(jac_row_idx(map_ok), (jac_col_idx(map_ok)-1)*7+5, (rgb-2)*pimg_jac_z(map_ok), n_px, n_part*7, n_ok);
                else
                    big_jac_dx      = sparse(n_px, n_part*7);
                    big_jac_dy      = sparse(n_px, n_part*7);
                    big_jac_dz      = sparse(n_px, n_part*7);
                end
                big_jac				= big_jac ...
                                    + big_jac_x + big_jac_y + big_jac_z ...
                                    + big_jac_dx + big_jac_dy + big_jac_dz ...
                                    + big_jac_D;
            end
            
            %% compute moment matrix, J'J
            big_JtJ				= big_jac.' * big_jac;
            % accumulate system jacobian 
            JtJ					= JtJ  + big_JtJ;

            %% compute J' * (y - y^)
            % this is the term on the RHS of the Levenberg-Marquard update
            big_JtDy			= big_jac.' * img_res_it0{ic}(:);
            % accumulate residual
            JtDy				= JtDy + big_JtDy(:);
		end
		JtJ_bw					= bandwidth(JtJ);
		Jtj_bandden				= JtJ_bw / size(JtJ,1);
		fprintf('shake%02d JtJ sparsity = %0.4f%%\n', n, 100*nnz(JtJ)/numel(JtJ));
		fprintf('shake%02d JtJ bandwidth= %5d (den=%0.3f%%)\n', n, JtJ_bw, 100*Jtj_bandden);
		
		%% update model
		% given residual and Jacobian, we calculate an update to the position
		% velocity and intensity (diameter^2) of each particle
		if opts.b_verbose
			fprintf('shake%02d shaking the box!\n', n);
		end

		% add Levenberg-Marquardt parameter
		lambda_rep				= repelem(ipr0.lambda(:), 7);
		JtJ_diag				= diag(diag(JtJ).*lambda_rep);
		JtJ_plus_JtJ_lambda     = JtJ + JtJ_diag;
		% solve for change in model parameter
		delta_beta				= JtJ_plus_JtJ_lambda \ JtDy;
		% extract model parameter
		delta_d                 = delta_beta(1:7:end,1)';
        delta_pos				= [delta_beta(2:7:end,1) delta_beta(3:7:end,1) delta_beta(4:7:end,1)]';
		delta_dx				= [delta_beta(5:7:end,1) delta_beta(6:7:end,1) delta_beta(7:7:end,1)]';
		
		% limit maximum change in position to dx_max_step
		delta_pos_len			= sqrt(sum(delta_pos.^2,1));
		delta_pos_hat			= bsxfun(@rdivide, delta_pos, delta_pos_len);
		delta_pos				= bsxfun(@times, delta_pos_hat, min(delta_pos_len, dx_max_step));
		
        % create update
        ipr1                    = ipr0;
        ipr1.re_part            = ipr1.re_part + delta_pos;
        ipr1.dx_part            = ipr1.dx_part + delta_dx;
        ipr1.d_part             = ipr1.d_part  + delta_d;
		
		% constrain shift
		% we do not move position beyond bounds set at the beginning
        ipr1.re_part            = max(ipr1.re_part, ipr1.re_part_lb);
        ipr1.re_part            = min(ipr1.re_part, ipr1.re_part_ub);
        % displacement is constrained by opts.dx_lim
		ipr1.dx_part            = max(ipr1.dx_part, opts.dx_lim(:,1)); % lower bound 
        ipr1.dx_part            = min(ipr1.dx_part, opts.dx_lim(:,2)); % upper bound
        
		%% evaluate residual of new system
        [img_proj_it1, pimg1]   = rgbipr_project(camera, otf, ipr1);
        % evaluate residual
        img_res_it1				= cell(1,n_cameras);
        for rgb = 1 : 3
            img_res_it1{rgb}    = img_ary{rgb} - img_proj_it1{rgb};
        end
        [ipr0, pimg0]           = rgbipr_residual(ipr0, pimg0, img_res_it0);
		[ipr1, pimg1]           = rgbipr_residual(ipr1, pimg1, img_res_it1);
        
		%% apply update of estimate
		% we only accept changes that reduce the residual in the particle
		% image. if the residual is improved above a certain threshold,
		% then the change is accepted. Otherwise, the change is rejected, and
		% the lambda parameter will be adjusted 
        
		% determine which shifts have improved fit
		b_accept					= ipr1.img_res <= ipr0.img_res;
		n_accept					= nnz(b_accept);
		
		% apply shifts that improve fit and recalculate lambda
		ipr1.re_part(:,~b_accept)    = ipr0.re_part(:,~b_accept);
        ipr1.dx_part(:,~b_accept)    = ipr0.dx_part(:,~b_accept);
		ipr1.d_part(:,~b_accept)     = ipr0.d_part(:,~b_accept);
        ipr1.img_res(:,~b_accept)    = ipr0.img_res(:,~b_accept);
        ipr1.lambda(:,b_accept)      = max(opts.lambda_min, ipr0.lambda(:, b_accept) / opts.Ldown);
		ipr1.lambda(:,~b_accept)	 = min(opts.lambda_max, ipr0.lambda(:,~b_accept) * opts.Lup  );
		
		if opts.b_verbose
			fprintf('shake%02d accepted %d of %d\n', n, n_accept, n_part);
		end
		
		%% report norm of residual
		if opts.b_verbose
			fprintf('shake%02d RMS    residual %0.3f counts per voxel\n', n, sqrt(mean(ipr1.img_res)));
			fprintf('shake%02d median residual %0.3f counts per voxel\n', n, median(sqrt(ipr1.img_res)));
		end
		
		%% convergence test
		delta_norm                  = ipr1.img_res - ipr0.img_res;
        mean_norm                   = mean(ipr1.img_norm, 2);
		b_converged                 = abs(delta_norm) < opts.tol_converged * mean_norm;
		if all(b_converged)
			if opts.b_verbose
				fprintf('Terminating due to convergence condition\n');
			end
			break;
		end
		
		%% remove close particles
        % and set this as the next iteration
		ipr1                = rgbipr_remove(ipr1, opts.R_nn_obj);
        ipr0                = ipr1;
    end
	
    %% evaluate residual of final system
    [img_proj_it1, pimg1]   = rgbipr_project(camera, otf, ipr1);
    % evaluate residual
    img_res_it1				= cell(1,n_cameras);
    for rgb = 1 : 3
        img_res_it1{rgb}    = img_ary{rgb} - img_proj_it1{rgb};
    end
    [ipr1, pimg1]           = rgbipr_residual(ipr1, pimg1, img_res_it1);
    % compute signal to noise ratio
    ipr1.snr                = 10*log10(ipr1.img_res ./ ipr1.img_norm);
end

