function [PSF, im_sharp] = psf_estimate_local(x, y, im, dx, dy, ksize, lambda)
	% psf_estimate_local(x, y, im, dx, dy, ksize, lambda)
	%
	% estimate PSF based on the method outlined in 
	%	PSF Estimation using Sharp Edge Prediction
	% Neel Joshi† Richard Szeliski∗ David J. Kriegman
	%
	% Input:
	%	x		x coordinate of image
	%	y		y coordinate of image
	%   im		image of checkerboard calibration target
	%	dx		x spacing of squares
	%	dy		y spacing of squares
	%	ksize	kernel size
	%	lambda  regularisation strength parameter
	% 
	% Output:
	%	PSF		estimated PSF ksize x ksize

	if nargin < 7
		lambda		= 2;	% regularisation strength parameter
	end

	%% create synthetic checkerboard matrix
	[xm,ym]		= ndgrid(x, y);
	% create synthetic checkerboard
	im_check_ref= xor(mod(xm, abs(dx)*2) >= abs(dx), ...
				  	  mod(ym, abs(dy)*2) >= abs(dy));
	% label regions on the checkerboard
	[im_check_w,n_white]= bwlabel( im_check_ref,4);
	[im_check_b,n_black]= bwlabel(~im_check_ref,4);
	% create index for all tiles
	im_check_b  = im_check_b + n_white;
	im_check_b(im_check_ref) = 0;
	im_check_ind= im_check_w+ im_check_b;
	n_tiles		= n_white + n_black;
	% average value over checkerboard tile
	im_bw_val   = accumarray(im_check_ind(:), im(:), [n_tiles 1], @mean);
	% create reference "sharp" image
	im_sharp    = im_bw_val(im_check_ind);

	%% build convolution matrix
	% represent convolution operation as matrix multiplication
	% C * PSF(:) = im(:)
	n_px		= numel(im);
	n_psf		= ksize^2;
	C			= zeros(n_px, n_psf);
	Kx			= zeros(n_psf, n_psf);
	Ky			= zeros(n_psf, n_psf);
	for n = 1 : n_psf
		% represent convolution operation in matrix
		PSF		= zeros(ksize, ksize);
		PSF(n)  = 1;
		im_conv = convn(im_sharp, PSF, 'same');
		C(:,n)  = im_conv(:);
		% represent gradient penalty
		[ky,kx] = gradient(PSF);
		Kx(:,n) = kx(:);
		Ky(:,n) = ky(:);
	end

	%% check
	% check convolution matrix constructed correctly
% 	PSF  = randn(ksize);
% 	im_conv_C   = C * PSF(:);
% 	im_conv_ref = convn(im_sharp, PSF, 'same');
% 	disp(norm(im_conv_ref(:)-im_conv_C(:)))

	%% estimate noise
	sigma		= estimate_noise(im);

	%% build NNLS system
	% express least squares minimisation of
	%	|im - convn(im_sharp, PSF)|^2 + sigma^2*lambda*gamma*|grad PSF|^2
	% i.e. equation 5 of [1]
	% in the form
	%	|C * k - d|^2

	% regularisation factor
	gamma		= ksize^2 * n_px;	% normalisation for kernel size and number of observations
	Gamma		= sqrt(lambda*gamma*sigma^2);
	C_full		= [C; Gamma*Kx; Gamma*Ky];
	d_full		= [im(:); zeros(n_psf*2,1)];

	%% solve regularised NNLS system
	opts		= optimset('Display', 'final');
	PSF			= lsqnonneg(C_full, d_full, opts);
	%PSF			= lsqlin(C_full, d_full);
	PSF			= reshape(PSF, ksize, ksize);
end

function Sigma=estimate_noise(I)
	% by Tolga Birdal
	% This is an extremely simple m-file which implements the method described
	% in : J. Immerk�r, �Fast Noise Variance Estimation�, Computer Vision and
	% Image Understanding, Vol. 64, No. 2, pp. 300-302, Sep. 1996
	% 
	% The function inputs a grayscale image I and returns Sigma, the noise
	% estimate. Here is a sample use: 
	% 
	% I = rgb2gray(imread('sample.jpg'));
	% Sigma=estimate_noise(I);
	%         
	% The advantage of this method is that it includes a Laplacian operation 
	% which is almost insensitive to image structure but only depends on the 
	% noise in the image. 


	[H, W]=size(I);
	I=double(I);
	% compute sum of absolute values of Laplacian
	M=[1 -2 1; -2 4 -2; 1 -2 1];
	Sigma=sum(sum(abs(conv2(I, M))));
	% scale sigma with proposed coefficients
	Sigma=Sigma*sqrt(0.5*pi)./(6*(W-2)*(H-2));
end